from common import TestMetaWearBase, to_string_buffer
from mbientlab.metawear.cbindings import *
from ctypes import create_string_buffer

class TestMma8452qConfiguration(TestMetaWearBase):
    def test_set_odr(self):
        expected= [0x03, 0x03, 0x00, 0x00, 0x20, 0x00, 0x00]

        self.libmetawear.mbl_mw_acc_mma8452q_set_odr(self.board, AccMma8452qOdr._50Hz)
        self.libmetawear.mbl_mw_acc_mma8452q_write_acceleration_config(self.board)
        self.assertListEqual(self.command, expected)

    def test_set_range(self):
        expected= [0x03, 0x03, 0x02, 0x00, 0x18, 0x00, 0x00]

        self.libmetawear.mbl_mw_acc_mma8452q_set_range(self.board, AccMma8452qRange._8G)
        self.libmetawear.mbl_mw_acc_mma8452q_write_acceleration_config(self.board)
        self.assertListEqual(self.command, expected)

    def test_set_odr_and_range(self):
        expected= [0x03, 0x03, 0x01, 0x00, 0x10, 0x00, 0x00]

        self.libmetawear.mbl_mw_acc_mma8452q_set_odr(self.board, AccMma8452qOdr._200Hz)
        self.libmetawear.mbl_mw_acc_mma8452q_set_range(self.board, AccMma8452qRange._4G)
        self.libmetawear.mbl_mw_acc_mma8452q_write_acceleration_config(self.board)
        self.assertListEqual(self.command, expected)

    def test_enable_acceleration_sampling(self):
        expected= [0x03, 0x02, 0x01]

        self.libmetawear.mbl_mw_acc_mma8452q_enable_acceleration_sampling(self.board)
        self.assertListEqual(self.command, expected)

    def test_disable_acceleration_sampling(self):
        expected= [0x03, 0x02, 0x00]

        self.libmetawear.mbl_mw_acc_mma8452q_disable_acceleration_sampling(self.board)
        self.assertListEqual(self.command, expected)

class TestAccMma8452qAccelerationData(TestMetaWearBase):
    def setUp(self):
        super().setUp()

        self.accel_data_signal= self.libmetawear.mbl_mw_acc_mma8452q_get_acceleration_data_signal(self.board)

    def test_get_acceleration_data_g(self):
        response= create_string_buffer(b'\x03\x04\x56\xfa\x05\xf6\x18\x03', 8)
        expected= CartesianFloat(x= -1.450, y= -2.555, z= 0.792)

        self.libmetawear.mbl_mw_datasignal_subscribe(self.accel_data_signal, None, self.sensor_data_handler)
        self.notify_mw_char(response)

        self.assertEqual(self.data_cartesian_float, expected)

    def test_handle_data_component(self):
        response= create_string_buffer(b'\x03\x04\x56\xfa\x05\xf6\x18\x03', 8)
        tests= [
            {
                'expected': -1.450,
                'index': Const.ACC_ACCEL_X_AXIS_INDEX,
                'name': 'x-axis'
            },
            {
                'expected': -2.555,
                'index': Const.ACC_ACCEL_Y_AXIS_INDEX,
                'name': 'y-axis'
            },
            {
                'expected': 0.792,
                'index': Const.ACC_ACCEL_Z_AXIS_INDEX,
                'name': 'z-axis'
            }
        ]

        for test in tests:
            with self.subTest(odr= test['name']):
                acc_component = self.libmetawear.mbl_mw_datasignal_get_component(self.accel_data_signal, test['index'])
                self.libmetawear.mbl_mw_datasignal_subscribe(acc_component, None, self.sensor_data_handler)
                self.notify_mw_char(response)
                
                self.assertAlmostEqual(self.data_float.value, test['expected'], delta = 0.001)

    def test_stream_acceleration_data(self):
        expected= [0x03, 0x04, 0x01]

        self.libmetawear.mbl_mw_datasignal_subscribe(self.accel_data_signal, None, self.sensor_data_handler)
        self.assertListEqual(self.command, expected)

    def test_end_stream_acceleration_data(self):
        expected= [0x03, 0x04, 0x00]

        self.libmetawear.mbl_mw_datasignal_unsubscribe(self.accel_data_signal)
        self.assertListEqual(self.command, expected)

class TestAccMma8452qHighFreqAccData(TestMetaWearBase):
    def setUp(self):
        super().setUp()

        self.accel_data_signal= self.libmetawear.mbl_mw_acc_mma8452q_get_packed_acceleration_data_signal(self.board)

    def sensorDataHandler(self, context, data):
        super().sensorDataHandler(context, data)

        self.cartesian_float_values.append(self.data_cartesian_float)

    def test_get_acceleration_data_g(self):
        response= create_string_buffer(b'\x03\x12\xdf\xed\x2b\x16\x15\xff\x7d\xfa\xed\x04\xc5\x0a\x31\xfb\xdb\xf8\x7b\xf2', 20)
        expected_values= [CartesianFloat(x= -4.641, y= 5.675, z= -0.235), CartesianFloat(x= -1.411, y= 1.261, z= 2.757), CartesianFloat(x= -1.231, y= -1.829, z= -3.461)]

        self.cartesian_float_values= []
        self.libmetawear.mbl_mw_datasignal_subscribe(self.accel_data_signal, None, self.sensor_data_handler)
        self.notify_mw_char(response)

        self.assertEqual(self.cartesian_float_values, expected_values)

    def test_stream_acceleration_data(self):
        expected= [0x03, 0x12, 0x01]

        self.libmetawear.mbl_mw_datasignal_subscribe(self.accel_data_signal, None, self.sensor_data_handler)
        self.assertListEqual(self.command, expected)

    def test_end_stream_acceleration_data(self):
        expected= [0x03, 0x12, 0x00]

        self.libmetawear.mbl_mw_datasignal_unsubscribe(self.accel_data_signal)
        self.assertListEqual(self.command, expected)

class TestOrientation(TestMetaWearBase):
    def test_enable(self):
        expected= [
            [0x03, 0x09, 0x00, 0xc0, 0x0a, 0x44, 0x84],
            [0x03, 0x08, 0x01]
        ]

        self.libmetawear.mbl_mw_acc_mma8452q_enable_orientation_detection(self.board)
        self.assertEqual(self.command_history, expected)

    def test_disable(self):
        expected= [
            [0x03, 0x08, 0x00],
            [0x03, 0x09, 0x00, 0x80, 0x00, 0x44, 0x84]
        ]

        self.libmetawear.mbl_mw_acc_mma8452q_disable_orientation_detection(self.board)
        self.assertEqual(self.command_history, expected)

    def test_handle_response(self):
        tests= [
            {
                'expected': SensorOrientation.FACE_UP_LANDSCAPE_RIGHT,
                'response': [0x03, 0x0a, 0x84],
                'name': 'fulr'
            },
            {
                'expected': SensorOrientation.FACE_UP_PORTRAIT_UPRIGHT,
                'response': [0x03, 0x0a, 0x80],
                'name': 'fupu'
            },
            {
                'expected': SensorOrientation.FACE_UP_PORTRAIT_UPSIDE_DOWN,
                'response': [0x03, 0x0a, 0x82],
                'name': 'fupd'
            },
            {
                'expected': SensorOrientation.FACE_UP_LANDSCAPE_LEFT,
                'response': [0x03, 0x0a, 0x86],
                'name': 'full'
            },
            {
                'expected': SensorOrientation.FACE_DOWN_LANDSCAPE_RIGHT,
                'response': [0x03, 0x0a, 0x85],
                'name': 'fdlr'
            },
            {
                'expected': SensorOrientation.FACE_DOWN_LANDSCAPE_LEFT,
                'response': [0x03, 0x0a, 0x87],
                'name': 'fdll'
            },
            {
                'expected': SensorOrientation.FACE_DOWN_PORTRAIT_UPRIGHT,
                'response': [0x03, 0x0a, 0x81],
                'name': 'fdpu'
            },
            {
                'expected': SensorOrientation.FACE_DOWN_PORTRAIT_UPSIDE_DOWN,
                'response': [0x03, 0x0a, 0x03],
                'name': 'fdpd'
            }
        ]

        for test in tests:
            with self.subTest(odr= test['name']):
                orientation = self.libmetawear.mbl_mw_acc_mma8452q_get_orientation_detection_data_signal(self.board)
                self.libmetawear.mbl_mw_datasignal_subscribe(orientation, None, self.sensor_data_handler)
                
                self.notify_mw_char(to_string_buffer(test['response']))
                self.assertEqual(self.data_int32.value, test['expected'])

class TestHighPassFilter(TestMetaWearBase):
    ODR = [
        AccMma8452qOdr._800Hz,
        AccMma8452qOdr._400Hz,
        AccMma8452qOdr._200Hz,
        AccMma8452qOdr._100Hz,
        AccMma8452qOdr._50Hz,
        AccMma8452qOdr._12_5Hz,
        AccMma8452qOdr._6_25Hz,
        AccMma8452qOdr._1_56Hz
    ]
    ODR_MASKS = [
        0x0, 0x8, 0x10, 0x18, 0x20, 0x28, 0x30, 0x38
    ]
    
    def test_configure_normal(self):
        params = [
            [[18.0, 15.9, 12.0, 3.125, 1.625, 2.125, 2.0, 1.5625], 0 ],
            [[10.0, 7.9, 5.75, 2.125, 1.375, 1.25, 0.9, 0.75], 1],
            [[5.0, 3.9, 1.75, 1.125, 0.375, 0.6, 0.74, 0.4], 2 ],
            [[2.75, 1.9, 0.75, 0.125, 0.374, 0.3, 0.20, 0.1], 3]
        ]
        tests = []
        for dr in TestHighPassFilter.ODR:
            for p in params:
                tests.append({
                    'odr': dr,
                    'cutoffs': p[0],
                    'mask': p[1]
                })

        for test in tests:
            with self.subTest(odr= test['odr']):
                self.libmetawear.mbl_mw_acc_mma8452q_set_odr(self.board, test['odr'])
                self.libmetawear.mbl_mw_acc_mma8452q_set_high_pass_cutoff(self.board, test['cutoffs'][test['odr']])
                self.libmetawear.mbl_mw_acc_mma8452q_write_acceleration_config(self.board)

                self.assertEqual(self.command, [0x03, 0x03, 0x10, test['mask'], TestHighPassFilter.ODR_MASKS[test['odr']], 0x00, 0x00])


from .cbindings import *
from ctypes import CDLL

import os
import platform

if (platform.system() == 'Windows'):
    _so_path = os.path.join(os.path.dirname(__file__), 'MetaWear.Win32.dll')
elif (platform.system() == 'Linux'):
    _so_path = os.path.join(os.path.dirname(__file__), 'libmetawear.so')
else:
    raise RuntimeError("MetaWear Python SDK is not supported for '%s'" % platform.system())

libmetawear= CDLL(_so_path)
init_libmetawear(libmetawear)

from .metawear import MetaWear

_value_parsers = {
    DataTypeId.UINT32: lambda p: cast(p.contents.value, POINTER(c_uint)).contents.value,
    DataTypeId.INT32: lambda p: cast(p.contents.value, POINTER(c_int)).contents.value,
    DataTypeId.FLOAT: lambda p: cast(p.contents.value, POINTER(c_float)).contents.value,
    DataTypeId.CARTESIAN_FLOAT: lambda p: cast(p.contents.value, POINTER(CartesianFloat)).contents,
    DataTypeId.BATTERY_STATE: lambda p: cast(p.contents.value, POINTER(BatteryState)).contents,
    DataTypeId.TCS34725_ADC: lambda p: cast(p.contents.value, POINTER(Tcs34725ColorAdc)).contents,
    DataTypeId.EULER_ANGLE: lambda p: cast(p.contents.value, POINTER(EulerAngles)).contents,
    DataTypeId.QUATERNION: lambda p: cast(p.contents.value, POINTER(Quaternion)).contents,
    DataTypeId.CORRECTED_CARTESIAN_FLOAT: lambda p: cast(p.contents.value, POINTER(CorrectedCartesianFloat)).contents,
    DataTypeId.OVERFLOW_STATE: lambda p: cast(p.contents.value, POINTER(OverflowState)).contents,
    DataTypeId.LOGGING_TIME: lambda p: cast(p.contents.value, POINTER(LoggingTime)).contents,
    DataTypeId.BTLE_ADDRESS: lambda p: cast(p.contents.value, POINTER(BtleAddress)).contents,
    DataTypeId.BOSCH_ANY_MOTION: lambda p: cast(p.contents.value, POINTER(BoschAnyMotion)).contents,
    DataTypeId.CALIBRATION_STATE: lambda p: cast(p.contents.value, POINTER(CalibrationState)).contents,
    DataTypeId.BOSCH_TAP: lambda p: cast(p.contents.value, POINTER(BoschTap)).contents
}
def parse_value(pointer, **kwargs):
    """
    Helper function to extract the value from a Data object.  If you are storing the values to be used at a later time, 
    call copy.deepcopy preserve the value.  You do not need to do this if the underlying type is a native type or a byte array
    @params:
        pointer     - Required  : Pointer to a Data object
        n_elem      - Optional  : Nummber of elements in the value array if the type_id attribute is DataTypeId.DATA_ARRAY
    """
    if (pointer.contents.type_id in _value_parsers):
        return _value_parsers[pointer.contents.type_id](pointer)
    elif (pointer.contents.type_id == DataTypeId.SENSOR_ORIENTATION):
        return _value_parsers[DataTypeId.INT32](pointer)
    elif (pointer.contents.type_id == DataTypeId.BYTE_ARRAY):
        array_ptr= cast(pointer.contents.value, POINTER(c_ubyte * pointer.contents.length))
        return [array_ptr.contents[i] for i in range(0, pointer.contents.length)]
    elif (pointer.contents.type_id == DataTypeId.DATA_ARRAY):
        if 'n_elem' in kwargs:
            values = cast(pointer.contents.value, POINTER(POINTER(Data) * kwargs['n_elem']))
            return [parse_value(values.contents[i]) for i in range(0, kwargs['n_elem'])]
        else:
            raise RuntimeError("Missing optional parameter 'n_elem' for parsing DataTypeId.DATA_ARRAY value")
    else:
        raise RuntimeError('Unrecognized data type id: ' + str(pointer.contents.type_id))



# usage: python led.py [mac]
from mbientlab.metawear import MetaWear, libmetawear
from mbientlab.metawear.cbindings import *
from time import sleep
from threading import Event

import sys

device = MetaWear(sys.argv[1])
device.connect()
print("Connected")

pattern= LedPattern(repeat_count= Const.LED_REPEAT_INDEFINITELY)
libmetawear.mbl_mw_led_load_preset_pattern(byref(pattern), LedPreset.SOLID)
libmetawear.mbl_mw_led_write_pattern(device.board, byref(pattern), LedColor.GREEN)
libmetawear.mbl_mw_led_play(device.board)

sleep(5.0)

libmetawear.mbl_mw_led_stop_and_clear(device.board)
sleep(1.0)

device.disconnect()
sleep(1.0)



from common import TestMetaWearBase
from ctypes import c_float, create_string_buffer
from mbientlab.metawear.cbindings import *

class TestColorDetectorTcs34725Config(TestMetaWearBase):
    def setUp(self):
        self.boardType= TestMetaWearBase.METAWEAR_ENV_BOARD

        super().setUp()

    def test_gain(self):
        tests= [
            {
                'expected': [0x17, 0x02, 0xff, 0x00, 0x00],
                'gain': ColorDetectorTcs34725Gain._1X,
                'gain_name': '1x'
            },
            {
                'expected': [0x17, 0x02, 0xff, 0x01, 0x00],
                'gain': ColorDetectorTcs34725Gain._4X,
                'gain_name': '4x'
            },
            {
                'expected': [0x17, 0x02, 0xff, 0x02, 0x00],
                'gain': ColorDetectorTcs34725Gain._16X,
                'gain_name': '16x'
            },
            {
                'expected': [0x17, 0x02, 0xff, 0x03, 0x00],
                'gain': ColorDetectorTcs34725Gain._60X,
                'gain_name': '60x'
            },
        ]

        for test in tests:
            with self.subTest(channel= test['gain_name']):
                self.libmetawear.mbl_mw_cd_tcs34725_set_gain(self.board, test['gain'])
                self.libmetawear.mbl_mw_cd_tcs34725_write_config(self.board)
                self.assertEqual(self.command, test['expected'])

    def test_integration_time(self):
        tests= [
            {
                'expected': [0x17, 0x02, 0x01, 0x00, 0x00],
                'integration_time': c_float(612),
                'time': '4.8ms'
            },
            {
                'expected': [0x17, 0x02, 0xfe, 0x00, 0x00],
                'integration_time': c_float(4.8),
                'time': '612ms'
            }
        ]

        for test in tests:
            with self.subTest(channel= test['time']):
                self.libmetawear.mbl_mw_cd_tcs34725_set_integration_time(self.board, test['integration_time'])
                self.libmetawear.mbl_mw_cd_tcs34725_write_config(self.board)
                self.assertEqual(self.command, test['expected'])

    def test_illuminator_enable(self):
        expected= [0x17, 0x02, 0xff, 0x00, 0x01]

        self.libmetawear.mbl_mw_cd_tcs34725_enable_illuminator_led(self.board)
        self.libmetawear.mbl_mw_cd_tcs34725_write_config(self.board)
        self.assertEqual(self.command, expected)

    def test_illuminator_enable(self):
        expected= [0x17, 0x02, 0xff, 0x00, 0x00]

        self.libmetawear.mbl_mw_cd_tcs34725_disable_illuminator_led(self.board)
        self.libmetawear.mbl_mw_cd_tcs34725_write_config(self.board)
        self.assertEqual(self.command, expected)

class TestColorDetectorTcs34725Data(TestMetaWearBase):
    def setUp(self):
        self.boardType= TestMetaWearBase.METAWEAR_ENV_BOARD

        super().setUp()

        self.adc_signal= self.libmetawear.mbl_mw_cd_tcs34725_get_adc_data_signal(self.board)

    def test_read_adc(self):
        expected= [0x17, 0x81]

        self.libmetawear.mbl_mw_datasignal_subscribe(self.adc_signal, None, self.sensor_data_handler)
        self.libmetawear.mbl_mw_datasignal_read(self.adc_signal)
        self.assertEqual(self.command, expected)

    def test_read_adc_silent(self):
        expected= [0x17, 0xc1]

        self.libmetawear.mbl_mw_datasignal_read(self.adc_signal)
        self.assertEqual(self.command, expected)

    def test_color_adc_data(self):
        expected= Tcs34725ColorAdc(clear= 418, red= 123, green= 154, blue= 124)
        response= create_string_buffer(b'\x17\x81\xa2\x01\x7b\x00\x9a\x00\x7c\x00', 10)

        self.libmetawear.mbl_mw_datasignal_subscribe(self.adc_signal, None, self.sensor_data_handler)
        self.notify_mw_char(response)

        self.assertEqual(self.data_tcs34725_adc, expected)

    def test_color_adc_component_data(self):
        response= create_string_buffer(b'\x17\x81\xa2\x01\x7b\x00\x9a\x00\x7c\x00', 10)
        tests= [
            {
                'expected': 418,
                'index': Const.CD_TCS34725_ADC_CLEAR_INDEX,
                'name': 'clear'
            },
            {
                'expected': 123,
                'index': Const.CD_TCS34725_ADC_RED_INDEX,
                'name': 'red'
            },
            {
                'expected': 154,
                'index': Const.CD_TCS34725_ADC_GREEN_INDEX,
                'name': 'green'
            },
            {
                'expected': 124,
                'index': Const.CD_TCS34725_ADC_BLUE_INDEX,
                'name': 'blue'
            }
        ]

        for test in tests:
            with self.subTest(odr= test['name']):
                signal_component = self.libmetawear.mbl_mw_datasignal_get_component(self.adc_signal, test['index'])

                self.libmetawear.mbl_mw_datasignal_subscribe(signal_component, None, self.sensor_data_handler)
                self.notify_mw_char(response) 

                self.assertEqual(self.data_uint32.value, test['expected'])



# usage: python scan_connect.py
from mbientlab.metawear import MetaWear
from mbientlab.metawear.cbindings import *
from mbientlab.warble import * 
from time import sleep

import platform
import six

selection = -1
devices = None

while selection == -1:
    print("scanning for devices...")
    devices = {}
    def handler(result):
        devices[result.mac] = result.name

    BleScanner.set_handler(handler)
    BleScanner.start()

    sleep(10.0)
    BleScanner.stop()

    i = 0
    for address, name in six.iteritems(devices):
        print("[%d] %s (%s)" % (i, address, name))
        i+= 1

    msg = "Select your device (-1 to rescan): "
    selection = int(raw_input(msg) if platform.python_version_tuple()[0] == '2' else input(msg))

address = list(devices)[selection]
print("Connecting to %s..." % (address))
device = MetaWear(address)
device.connect()

print("Connected")
print("Device information: " + str(device.info))
sleep(5.0)

device.disconnect()
sleep(1.0)
print("Disconnected") 



from common import TestMetaWearBase
from mbientlab.metawear.cbindings import *
from ctypes import *

class TestALsLtr329Config(TestMetaWearBase):
    def setUp(self):
        self.boardType= TestMetaWearBase.METAWEAR_RPRO_BOARD

        super().setUp()

    def test_set_gain(self):
        expected= [0x14, 0x02, 0x18, 0x03]

        self.libmetawear.mbl_mw_als_ltr329_set_gain(self.board, AlsLtr329Gain._48X)
        self.libmetawear.mbl_mw_als_ltr329_write_config(self.board)
        self.assertListEqual(self.command, expected)

    def test_set_integration_time(self):
        expected= [0x14, 0x02, 0x00, 0x1b]

        self.libmetawear.mbl_mw_als_ltr329_set_integration_time(self.board, AlsLtr329IntegrationTime._400ms)
        self.libmetawear.mbl_mw_als_ltr329_write_config(self.board)
        self.assertListEqual(self.command, expected)

    def test_set_measurement_rate(self):
        expected= [0x14, 0x02, 0x00, 0x05]

        self.libmetawear.mbl_mw_als_ltr329_set_measurement_rate(self.board, AlsLtr329MeasurementRate._2000ms)
        self.libmetawear.mbl_mw_als_ltr329_write_config(self.board)
        self.assertListEqual(self.command, expected)

    def test_set_all_config(self):
        expected= [0x14, 0x02, 0x0c, 0x28]

        self.libmetawear.mbl_mw_als_ltr329_set_gain(self.board, AlsLtr329Gain._8X)
        self.libmetawear.mbl_mw_als_ltr329_set_integration_time(self.board, AlsLtr329IntegrationTime._250ms)
        self.libmetawear.mbl_mw_als_ltr329_set_measurement_rate(self.board, AlsLtr329MeasurementRate._50ms)
        self.libmetawear.mbl_mw_als_ltr329_write_config(self.board)
        self.assertListEqual(self.command, expected)

class TestAlsLtr329DataHandler(TestMetaWearBase):
    def setUp(self):
        self.boardType= TestMetaWearBase.METAWEAR_RPRO_BOARD

        super().setUp()

        self.ltr329_data_signal= self.libmetawear.mbl_mw_als_ltr329_get_illuminance_data_signal(self.board)

    def test_get_illuminance_data(self):
        response= create_string_buffer(b'\x14\x03\xed\x92\xb0\x00', 6)
        expected= 11571949

        self.libmetawear.mbl_mw_datasignal_subscribe(self.ltr329_data_signal, None, self.sensor_data_handler)
        self.notify_mw_char(response)
        self.assertEqual(self.data_uint32.value, expected)

    def test_stream_illuminance_data(self):
        expected= [0x14, 0x03, 0x01]

        self.libmetawear.mbl_mw_datasignal_subscribe(self.ltr329_data_signal, None, self.sensor_data_handler)
        self.assertListEqual(self.command, expected)

    def test_end_stream_illuminance_data(self):
        expected= [0x14, 0x03, 0x00]

        self.libmetawear.mbl_mw_datasignal_unsubscribe(self.ltr329_data_signal)
        self.assertListEqual(self.command, expected)



import copy
from common import TestMetaWearBase
from ctypes import create_string_buffer
from mbientlab.metawear.cbindings import *

class TestGyroBmi160Config(TestMetaWearBase):
    def setUp(self):
        self.boardType= TestMetaWearBase.METAWEAR_RG_BOARD

        super().setUp()

    def test_mbl_mw_gyro_bmi160_set_odr(self):
        expected= [0x13, 0x3, 0x29, 0x0]

        self.libmetawear.mbl_mw_gyro_bmi160_set_odr(self.board, GyroBmi160Odr._200Hz)
        self.libmetawear.mbl_mw_gyro_bmi160_write_config(self.board)
        self.assertListEqual(self.command, expected)

    def test_mbl_mw_gyro_bmi160_set_fsr(self):
        expected= [0x13, 0x03, 0x28, 0x03]

        self.libmetawear.mbl_mw_gyro_bmi160_set_range(self.board, GyroBmi160Range._250dps)
        self.libmetawear.mbl_mw_gyro_bmi160_write_config(self.board)
        self.assertListEqual(self.command, expected)

    def test_mbl_mw_gyro_bmi160_set_all_config(self):
        expected= [0x13, 0x03, 0x27, 0x04]

        self.libmetawear.mbl_mw_gyro_bmi160_set_odr(self.board, GyroBmi160Odr._50Hz)
        self.libmetawear.mbl_mw_gyro_bmi160_set_range(self.board, GyroBmi160Range._125dps)
        self.libmetawear.mbl_mw_gyro_bmi160_write_config(self.board)
        self.assertListEqual(self.command, expected)

    def test_gyro_active(self):
        expected= [0x13, 0x01, 0x01]

        self.libmetawear.mbl_mw_gyro_bmi160_start(self.board)
        self.assertListEqual(self.command, expected)

    def test_gyro_standby(self):
        expected= [0x13, 0x01, 0x00]

        self.libmetawear.mbl_mw_gyro_bmi160_stop(self.board)
        self.assertListEqual(self.command, expected)

    def test_enable_rotation_sampling(self):
        expected= [0x13, 0x02, 0x01, 0x00]

        self.libmetawear.mbl_mw_gyro_bmi160_enable_rotation_sampling(self.board)
        self.assertListEqual(self.command, expected)

    def test_disable_rotation_sampling(self):
        expected= [0x13, 0x02, 0x00, 0x01]

        self.libmetawear.mbl_mw_gyro_bmi160_disable_rotation_sampling(self.board)
        self.assertListEqual(self.command, expected)

class TestGyroBmi160DataHandler(TestMetaWearBase):
    def setUp(self):
        self.boardType= TestMetaWearBase.METAWEAR_RG_BOARD

        super().setUp()

        self.gyro_rot_data_signal= self.libmetawear.mbl_mw_gyro_bmi160_get_rotation_data_signal(self.board)

    def test_subscribe_rotation_data(self):
        expected= [0x13, 0x05, 0x01]

        self.libmetawear.mbl_mw_datasignal_subscribe(self.gyro_rot_data_signal, None, self.sensor_data_handler)
        self.assertListEqual(self.command, expected)

    def test_unsubscribe_rotation_data(self):
        expected= [0x13, 0x05, 0x00]

        self.libmetawear.mbl_mw_datasignal_unsubscribe(self.gyro_rot_data_signal)
        self.assertListEqual(self.command, expected)

    def test_rotation_data_handler(self):
        expected= CartesianFloat(x= 262.409, y= 499.497, z= -499.512)

        self.libmetawear.mbl_mw_datasignal_subscribe(self.gyro_rot_data_signal, None, self.sensor_data_handler)
        self.libmetawear.mbl_mw_gyro_bmi160_set_range(self.board, GyroBmi160Range._500dps)
        self.notify_mw_char(create_string_buffer(b'\x13\x05\x3e\x43\xff\x7f\x00\x80', 8))

        self.assertEqual(self.data_cartesian_float, expected)

    def test_rotation_data_component_handler(self):
        response= create_string_buffer(b'\x13\x05\x3e\x43\xff\x7f\x00\x80', 8)
        tests= [
            {
                'expected': 262.409,
                'index': Const.GYRO_ROTATION_X_AXIS_INDEX,
                'name': 'x-axis'
            },
            {
                'expected': 499.497,
                'index': Const.GYRO_ROTATION_Y_AXIS_INDEX,
                'name': 'y-axis'
            },
            {
                'expected': -499.512,
                'index': Const.GYRO_ROTATION_Z_AXIS_INDEX,
                'name': 'z-axis'
            }
        ]

        for test in tests:
            with self.subTest(odr= test['name']):
                rot_component = self.libmetawear.mbl_mw_datasignal_get_component(self.gyro_rot_data_signal, test['index'])
                self.libmetawear.mbl_mw_datasignal_subscribe(rot_component, None, self.sensor_data_handler)
                self.libmetawear.mbl_mw_gyro_bmi160_set_range(self.board, GyroBmi160Range._500dps)
                self.notify_mw_char(response)
                
                self.assertAlmostEqual(self.data_float.value, test['expected'], delta = 0.001)

class TestGyroBmi160HighFreqDataHandler(TestMetaWearBase):
    def setUp(self):
        self.boardType= TestMetaWearBase.METAWEAR_RG_BOARD

        super().setUp()

        self.gyro_rot_data_signal= self.libmetawear.mbl_mw_gyro_bmi160_get_packed_rotation_data_signal(self.board)

    def sensorDataHandler(self, context, data):
        super().sensorDataHandler(context, data)

        self.cartesian_float_values.append(self.data_cartesian_float)

    def test_subscribe_rotation_data(self):
        expected= [0x13, 0x07, 0x01]

        self.libmetawear.mbl_mw_datasignal_subscribe(self.gyro_rot_data_signal, None, self.sensor_data_handler)
        self.assertListEqual(self.command, expected)

    def test_unsubscribe_rotation_data(self):
        expected= [0x13, 0x07, 0x00]

        self.libmetawear.mbl_mw_datasignal_unsubscribe(self.gyro_rot_data_signal)
        self.assertListEqual(self.command, expected)

    def test_rotation_data_handler(self):
        expected_values= [CartesianFloat(x= 164.177, y= 301.860, z= -265.122), CartesianFloat(x= 207.134, y= 296.982, z= -215.457), CartesianFloat(x= 254.055, y= 309.634, z= -152.591)]

        self.cartesian_float_values= []
        self.libmetawear.mbl_mw_datasignal_subscribe(self.gyro_rot_data_signal, None, self.sensor_data_handler)
        self.libmetawear.mbl_mw_gyro_bmi160_set_range(self.board, GyroBmi160Range._1000dps)
        self.notify_mw_char(create_string_buffer(b'\x13\x07\x09\x15\xad\x26\x08\xde\x8a\x1a\x0d\x26\x65\xe4\x8d\x20\xac\x27\x73\xec', 20))

        self.assertEqual(self.cartesian_float_values, expected_values)








from common import TestMetaWearBase
from ctypes import byref
from mbientlab.metawear.cbindings import *
import threading

class TestMacro(TestMetaWearBase):
    def setUp(self):
        self.boardType= TestMetaWearBase.METAWEAR_RPRO_BOARD

        super().setUp()

    def test_led_on_boot(self):
        e = threading.Event()
        callback = FnVoid_VoidP_VoidP_Int(lambda ctx, board, status: e.set())

        pattern= LedPattern(rise_time_ms = 0, pulse_duration_ms = 1000, repeat_count = 5, high_time_ms = 500, high_intensity = 16, low_intensity = 16)

        self.libmetawear.mbl_mw_macro_record(self.board, 1)
        self.libmetawear.mbl_mw_led_write_pattern(self.board, byref(pattern), LedColor.BLUE)
        self.libmetawear.mbl_mw_led_play(self.board)
        self.libmetawear.mbl_mw_macro_end_record(self.board, None, callback)

        e.wait()

        expected = [
            [0x02, 0x03, 0x02, 0x02, 0x10, 0x10, 0x00, 0x00, 0xf4, 0x01, 0x00, 0x00, 0xe8, 0x03, 0x00, 0x00, 0x05],
            [0x02, 0x01, 0x01],
            [0x0f, 0x02, 0x01],
            [0x0f, 0x03, 0x02, 0x03, 0x02, 0x02, 0x10, 0x10, 0x00, 0x00, 0xf4, 0x01, 0x00, 0x00, 0xe8, 0x03, 0x00, 0x00, 0x05],
            [0x0f, 0x03, 0x02, 0x01, 0x01],
            [0x0f, 0x04]
        ]
        self.assertEqual(self.command_history, expected)

    def test_freefall_on_boot(self):
        events = [threading.Event(), threading.Event()]

        def threshold_processor_created(context, pointer):
            threshold_signal= pointer
            self.comparator_below_handler = FnVoid_VoidP_VoidP(lambda ctx, pointer: self.libmetawear.mbl_mw_dataprocessor_comparator_create(threshold_signal, 
                ComparatorOperation.EQ, 1.0, ctx, comparator_above_handler))
            comparator_above_handler = FnVoid_VoidP_VoidP(lambda ctx, pointer: events[1].set())
            self.libmetawear.mbl_mw_dataprocessor_comparator_create(pointer, ComparatorOperation.EQ, -1.0, context, self.comparator_below_handler)

        rss_handler= FnVoid_VoidP_VoidP(lambda ctx, pointer: self.libmetawear.mbl_mw_dataprocessor_average_create(pointer, 4, ctx, avg_handler))
        avg_handler= FnVoid_VoidP_VoidP(lambda ctx, pointer: self.libmetawear.mbl_mw_dataprocessor_threshold_create(pointer, ThresholdMode.BINARY, 0.5, 0.0,  
                ctx, threshold_handler))
        threshold_handler= FnVoid_VoidP_VoidP(threshold_processor_created)
        callback = FnVoid_VoidP_VoidP_Int(lambda ctx, board, status: events[0].set())

        accel_signal= self.libmetawear.mbl_mw_acc_get_acceleration_data_signal(self.board)

        self.libmetawear.mbl_mw_macro_record(self.board, 1)
        self.libmetawear.mbl_mw_dataprocessor_rss_create(accel_signal, None, rss_handler)
        events[1].wait()

        self.libmetawear.mbl_mw_acc_enable_acceleration_sampling(self.board)
        self.libmetawear.mbl_mw_acc_start(self.board)

        self.libmetawear.mbl_mw_macro_end_record(self.board, None, callback)
        events[0].wait()

        expected = [
            [0x09, 0x02, 0x03, 0x04, 0xff, 0xa0, 0x07, 0xa5, 0x01],
            [0x09, 0x02, 0x09, 0x03, 0x00, 0x20, 0x03, 0x05, 0x04],
            [0x09, 0x02, 0x09, 0x03, 0x01, 0x20, 0x0d, 0x09, 0x00, 0x20, 0x00, 0x00, 0x00, 0x00],
            [0x09, 0x02, 0x09, 0x03, 0x02, 0x00, 0x06, 0x01, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff],
            [0x09, 0x02, 0x09, 0x03, 0x02, 0x00, 0x06, 0x01, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00],
            [0x03, 0x02, 0x01, 0x00],
            [0x03, 0x01, 0x01],
            [0x0f, 0x02, 0x01],
            [0x0f, 0x03, 0x09, 0x02, 0x03, 0x04, 0xff, 0xa0, 0x07, 0xa5, 0x01],
            [0x0f, 0x03, 0x09, 0x02, 0x09, 0x03, 0x00, 0x20, 0x03, 0x05, 0x04],
            [0x0f, 0x03, 0x09, 0x02, 0x09, 0x03, 0x01, 0x20, 0x0d, 0x09, 0x00, 0x20, 0x00, 0x00, 0x00, 0x00],
            [0x0f, 0x03, 0x09, 0x02, 0x09, 0x03, 0x02, 0x00, 0x06, 0x01, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff],
            [0x0f, 0x03, 0x09, 0x02, 0x09, 0x03, 0x02, 0x00, 0x06, 0x01, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00],
            [0x0f, 0x03, 0x03, 0x02, 0x01, 0x00],
            [0x0f, 0x03, 0x03, 0x01, 0x01],
            [0x0f, 0x04]
        ]
        self.maxDiff = None
        self.assertEqual(self.command_history, expected)

    def test_led_controller(self):
        control_signals = [None, None, None]
        events = [threading.Event(), threading.Event(), threading.Event(), threading.Event()]

        def comp_even_created(context, pointer):
            control_signals[1] = pointer
            events[0].set()

        def comp_odd_created(context, pointer):
            control_signals[0] = pointer
            self.libmetawear.mbl_mw_dataprocessor_comparator_create(control_signals[2], ComparatorOperation.EQ, 0.0, context, comparator_even_handler)

        def math_created(context, pointer):
            control_signals[2] = pointer
            self.libmetawear.mbl_mw_dataprocessor_comparator_create(pointer, ComparatorOperation.EQ, 1.0, context, comparator_odd_handler)

        math_handler = FnVoid_VoidP_VoidP(math_created)
        comparator_even_handler = FnVoid_VoidP_VoidP(comp_even_created)
        comparator_odd_handler = FnVoid_VoidP_VoidP(comp_odd_created)
        counter_handler = FnVoid_VoidP_VoidP(lambda ctx, p: self.libmetawear.mbl_mw_dataprocessor_math_create(p, MathOperation.MODULUS, 2.0, ctx, math_handler))

        switch_signal= self.libmetawear.mbl_mw_switch_get_state_data_signal(self.board)

        self.libmetawear.mbl_mw_macro_record(self.board, 0)
        self.libmetawear.mbl_mw_dataprocessor_counter_create(switch_signal, None, counter_handler)
        events[0].wait()

        event_handler = FnVoid_VoidP_VoidP_Int(lambda ctx, e, s: events[1].set())
        pattern= LedPattern(pulse_duration_ms=1000, high_time_ms=500, high_intensity=16, low_intensity=16, 
                repeat_count=Const.LED_REPEAT_INDEFINITELY)
        self.libmetawear.mbl_mw_event_record_commands(control_signals[0])
        self.libmetawear.mbl_mw_led_write_pattern(self.board, byref(pattern), LedColor.BLUE)
        self.libmetawear.mbl_mw_led_play(self.board)
        self.libmetawear.mbl_mw_event_end_record(control_signals[0], None, event_handler)
        events[1].wait()

        event_handler = FnVoid_VoidP_VoidP_Int(lambda ctx, e, s: events[2].set())
        self.libmetawear.mbl_mw_event_record_commands(control_signals[1])
        self.libmetawear.mbl_mw_led_stop_and_clear(self.board)
        self.libmetawear.mbl_mw_event_end_record(control_signals[1], None, event_handler)
        events[2].wait()

        callback = FnVoid_VoidP_VoidP_Int(lambda ctx, board, status: events[3].set())
        self.libmetawear.mbl_mw_macro_end_record(self.board, None, callback)
        events[3].wait()

        expected = [
            [0x09, 0x02, 0x01, 0x01, 0xff, 0x00, 0x02, 0x10],
            [0x09, 0x02, 0x09, 0x03, 0x00, 0x00, 0x09, 0x03, 0x04, 0x02, 0x00, 0x00, 0x00, 0x00],
            [0x09, 0x02, 0x09, 0x03, 0x01, 0x60, 0x06, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00],
            [0x09, 0x02, 0x09, 0x03, 0x01, 0x60, 0x06, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00],
            [0x0a, 0x02, 0x09, 0x03, 0x02, 0x02, 0x03, 0x0f],
            [0x0a, 0x03, 0x02, 0x02, 0x10, 0x10, 0x00, 0x00, 0xf4, 0x01, 0x00, 0x00, 0xe8, 0x03, 0x00, 0x00, 0xff],
            [0x0a, 0x02, 0x09, 0x03, 0x02, 0x02, 0x01, 0x01],
            [0x0a, 0x03, 0x01],
            [0x0a, 0x02, 0x09, 0x03, 0x03, 0x02, 0x02, 0x01],
            [0x0a, 0x03, 0x01],
            [0x0f, 0x02, 0x00],
            [0x0f, 0x03, 0x09, 0x02, 0x01, 0x01, 0xff, 0x00, 0x02, 0x10],
            [0x0f, 0x03, 0x09, 0x02, 0x09, 0x03, 0x00, 0x00, 0x09, 0x03, 0x04, 0x02, 0x00, 0x00, 0x00, 0x00],
            [0x0f, 0x03, 0x09, 0x02, 0x09, 0x03, 0x01, 0x60, 0x06, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00],
            [0x0f, 0x03, 0x09, 0x02, 0x09, 0x03, 0x01, 0x60, 0x06, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00],
            [0x0f, 0x03, 0x0a, 0x02, 0x09, 0x03, 0x02, 0x02, 0x03, 0x0f],
            [0x0f, 0x03, 0x0a, 0x03, 0x02, 0x02, 0x10, 0x10, 0x00, 0x00, 0xf4, 0x01, 0x00, 0x00, 0xe8, 0x03, 0x00, 0x00, 0xff],
            [0x0f, 0x03, 0x0a, 0x02, 0x09, 0x03, 0x02, 0x02, 0x01, 0x01],
            [0x0f, 0x03, 0x0a, 0x03, 0x01],
            [0x0f, 0x03, 0x0a, 0x02, 0x09, 0x03, 0x03, 0x02, 0x02, 0x01],
            [0x0f, 0x03, 0x0a, 0x03, 0x01],
            [0x0f, 0x04]
        ]
        self.assertEqual(self.command_history, expected)



from common import TestMetaWearBase
from mbientlab.metawear.cbindings import *
from ctypes import create_string_buffer

class TestAccBmi160Config(TestMetaWearBase):
    def setUp(self):
        self.boardType= TestMetaWearBase.METAWEAR_RG_BOARD

        super().setUp()

    def test_set_odr(self):
        tests= [
            {
                'expected': [0x03, 0x03, 0x81, 0x03],
                'odr': AccBmi160Odr._0_78125Hz,
                'odr_name': '0.78125Hz'
            },
            {
                'expected': [0x03, 0x03, 0x82, 0x03],
                'odr': AccBmi160Odr._1_5625Hz,
                'odr_name': '1.5625Hz'
            },
            {
                'expected': [0x03, 0x03, 0x83, 0x03],
                'odr': AccBmi160Odr._3_125Hz,
                'odr_name': '3.125Hz'
            },
            {
                'expected': [0x03, 0x03, 0x84, 0x03],
                'odr': AccBmi160Odr._6_25Hz,
                'odr_name': '6.25Hz'
            },
            {
                'expected': [0x03, 0x03, 0x25, 0x03],
                'odr': AccBmi160Odr._12_5Hz,
                'odr_name': '12.5Hz'
            },
            {
                'expected': [0x03, 0x03, 0x26, 0x03],
                'odr': AccBmi160Odr._25Hz,
                'odr_name': '25Hz'
            },
            {
                'expected': [0x03, 0x03, 0x27, 0x03],
                'odr': AccBmi160Odr._50Hz,
                'odr_name': '50Hz'
            },
            {
                'expected': [0x03, 0x03, 0x28, 0x03],
                'odr': AccBmi160Odr._100Hz,
                'odr_name': '100Hz'
            },
            {
                'expected': [0x03, 0x03, 0x29, 0x03],
                'odr': AccBmi160Odr._200Hz,
                'odr_name': '200Hz'
            },
            {
                'expected': [0x03, 0x03, 0x2a, 0x03],
                'odr': AccBmi160Odr._400Hz,
                'odr_name': '400Hz'
            },
            {
                'expected': [0x03, 0x03, 0x2b, 0x03],
                'odr': AccBmi160Odr._800Hz,
                'odr_name': '800Hz'
            },
            {
                'expected': [0x03, 0x03, 0x2c, 0x03],
                'odr': AccBmi160Odr._1600Hz,
                'odr_name': '1600Hz'
            }
        ]

        for test in tests:
            with self.subTest(odr= test['odr_name']):
                self.libmetawear.mbl_mw_acc_bmi160_set_odr(self.board, test['odr'])
                self.libmetawear.mbl_mw_acc_bosch_write_acceleration_config(self.board)
                self.assertListEqual(self.command, test['expected'])

    def test_set_range(self):
        expected= [0x03, 0x03, 0x28, 0x0c]

        self.libmetawear.mbl_mw_acc_bosch_set_range(self.board, AccBoschRange._16G)
        self.libmetawear.mbl_mw_acc_bosch_write_acceleration_config(self.board)
        self.assertListEqual(self.command, expected)

    def test_set_odr_and_range(self):
        expected= [0x03, 0x03, 0x29, 0x08]

        self.libmetawear.mbl_mw_acc_bmi160_set_odr(self.board, AccBmi160Odr._200Hz)
        self.libmetawear.mbl_mw_acc_bosch_set_range(self.board, AccBoschRange._8G)
        self.libmetawear.mbl_mw_acc_bosch_write_acceleration_config(self.board)
        self.assertListEqual(self.command, expected)

    def test_enable_acceleration_sampling(self):
        expected= [0x03, 0x02, 0x01, 0x00]

        self.libmetawear.mbl_mw_acc_bosch_enable_acceleration_sampling(self.board)
        self.assertListEqual(self.command, expected)

    def test_disable_acceleration_sampling(self):
        expected= [0x03, 0x02, 0x00, 0x01]

        self.libmetawear.mbl_mw_acc_bosch_disable_acceleration_sampling(self.board)
        self.assertListEqual(self.command, expected)

class TestBmi160AccelerationData(TestMetaWearBase):
    def setUp(self):
        self.boardType= TestMetaWearBase.METAWEAR_RG_BOARD

        super().setUp()

        self.accel_data_signal= self.libmetawear.mbl_mw_acc_bosch_get_acceleration_data_signal(self.board)

    def test_get_acceleration_data_g(self):
        response= create_string_buffer(b'\x03\x04\x16\xc4\x94\xa2\x2a\xd0')
        expected= CartesianFloat(x= -1.872, y= -2.919, z= -1.495)

        self.libmetawear.mbl_mw_datasignal_subscribe(self.accel_data_signal, None, self.sensor_data_handler)
        self.libmetawear.mbl_mw_acc_bosch_set_range(self.board, AccBoschRange._4G)
        self.notify_mw_char(response)
        self.assertEqual(self.data_cartesian_float, expected)

    def test_subscribe_acceleration_data(self):
        expected= [0x03, 0x04, 0x01]

        self.libmetawear.mbl_mw_datasignal_subscribe(self.accel_data_signal, None, self.sensor_data_handler)
        self.assertListEqual(self.command, expected)

    def test_unsubscribe_acceleration_data(self):
        expected= [0x03, 0x04, 0x00]

        self.libmetawear.mbl_mw_datasignal_unsubscribe(self.accel_data_signal)
        self.assertListEqual(self.command, expected)

class TestBmi160HighFreqAccData(TestMetaWearBase):
    def setUp(self):
        self.boardType= TestMetaWearBase.METAWEAR_RG_BOARD

        super().setUp()

        self.accel_data_signal= self.libmetawear.mbl_mw_acc_bosch_get_packed_acceleration_data_signal(self.board)

    def sensorDataHandler(self, context, data):
        super().sensorDataHandler(context, data)

        self.cartesian_float_values.append(self.data_cartesian_float)

    def test_get_acceleration_data_g(self):
        response= create_string_buffer(b'\x03\x1c\x62\xb7\x53\x0d\xe9\xfd\x16\xd0\x4d\x0e\x57\x02\x8a\xff\xa1\x05\x0a\x01', 20)
        expected_values= [CartesianFloat(x= -4.539, y= 0.833, z= -0.131), CartesianFloat(x= -2.995, y= 0.894, z= 0.146), CartesianFloat(x= -0.029, y= 0.352, z= 0.065)]

        self.cartesian_float_values= []
        self.libmetawear.mbl_mw_datasignal_subscribe(self.accel_data_signal, None, self.sensor_data_handler)
        self.libmetawear.mbl_mw_acc_bosch_set_range(self.board, AccBoschRange._8G)
        self.notify_mw_char(response)
        self.assertEqual(self.cartesian_float_values, expected_values)

    def test_subscribe(self):
        expected= [0x03, 0x1c, 0x01]

        self.libmetawear.mbl_mw_datasignal_subscribe(self.accel_data_signal, None, self.sensor_data_handler)
        self.assertListEqual(self.command, expected)

    def test_unsubscribe(self):
        expected= [0x03, 0x1c, 0x00]

        self.libmetawear.mbl_mw_datasignal_unsubscribe(self.accel_data_signal)
        self.assertListEqual(self.command, expected)

class TestBmi160StepCounterSetup(TestMetaWearBase):
    def setUp(self):
        self.boardType= TestMetaWearBase.METAWEAR_RG_BOARD

        super().setUp()

    def test_set_mode(self):
        tests= [
            {
                'expected': [0x03, 0x18, 0x15, 0x0b],
                'mode': AccBmi160StepCounterMode.NORMAL,
                'mode_name': 'normal'
            },
            {
                'expected': [0x03, 0x18, 0x2d, 0x08],
                'mode': AccBmi160StepCounterMode.SENSITIVE,
                'mode_name': 'sensitive'
            },
            {
                'expected': [0x03, 0x18, 0x1d, 0x0f],
                'mode': AccBmi160StepCounterMode.ROBUST,
                'mode_name': 'robust'
            }
        ]

        for test in tests:
            with self.subTest(mode= test['mode_name']):
                self.libmetawear.mbl_mw_acc_bmi160_set_step_counter_mode(self.board, test['mode'])
                self.libmetawear.mbl_mw_acc_bmi160_enable_step_counter(self.board)
                self.libmetawear.mbl_mw_acc_bmi160_write_step_counter_config(self.board)
                self.assertListEqual(self.command, test['expected'])

class TestBmi160StepCounterData(TestMetaWearBase):
    def setUp(self):
        self.boardType= TestMetaWearBase.METAWEAR_RG_BOARD

        super().setUp()

        self.step_counter_signal= self.libmetawear.mbl_mw_acc_bmi160_get_step_counter_data_signal(self.board)

    def test_get_step_count_value(self):
        response= create_string_buffer(b'\x03\x9a\x2b\x00', 4)
        expected= 43

        self.libmetawear.mbl_mw_datasignal_subscribe(self.step_counter_signal, None, self.sensor_data_handler)
        self.notify_mw_char(response)
        self.assertEqual(self.data_uint32.value, expected)

    def test_read_step_counter(self):
        expected= [0x03, 0x9a]

        self.libmetawear.mbl_mw_datasignal_subscribe(self.step_counter_signal, None, self.sensor_data_handler)
        self.libmetawear.mbl_mw_datasignal_read(self.step_counter_signal)
        self.assertListEqual(self.command, expected)

    def test_read_step_counter_silent(self):
        expected= [0x03, 0xda]

        self.libmetawear.mbl_mw_datasignal_read(self.step_counter_signal)
        self.assertListEqual(self.command, expected)

class TestBmi160StepDetectorData(TestMetaWearBase):
    def setUp(self):
        self.boardType= TestMetaWearBase.METAWEAR_RG_BOARD

        super().setUp()

        self.step_detector_signal= self.libmetawear.mbl_mw_acc_bmi160_get_step_detector_data_signal(self.board)

    def test_subscribe_detector(self):
        expected= [0x3, 0x19, 0x1]

        self.libmetawear.mbl_mw_datasignal_subscribe(self.step_detector_signal, None, self.sensor_data_handler)
        self.assertEqual(self.command, expected)

    def test_unsubscribe_detector(self):
        expected= [0x3, 0x19, 0x0]

        self.libmetawear.mbl_mw_datasignal_unsubscribe(self.step_detector_signal, self.sensor_data_handler)
        self.assertEqual(self.command, expected)

    def test_enable_detector(self):
        expected= [0x03, 0x17, 0x01, 0x00]

        self.libmetawear.mbl_mw_acc_bmi160_enable_step_detector(self.board)
        self.assertEqual(self.command, expected)

    def test_disable_detector(self):
        expected= [0x03, 0x17, 0x00, 0x01]

        self.libmetawear.mbl_mw_acc_bmi160_disable_step_detector(self.board)
        self.assertEqual(self.command, expected)

    def test_get_detection(self):
        response= create_string_buffer(b'\x03\x19\x01')
        expected= 1

        self.libmetawear.mbl_mw_datasignal_subscribe(self.step_detector_signal, None, self.sensor_data_handler)
        self.notify_mw_char(response)
        self.assertEqual(self.data_uint32.value, expected)



from common import TestMetaWearBase
from mbientlab.metawear.cbindings import *
import uuid

class TestIBeacon(TestMetaWearBase):
    def test_set_major(self):
        expected= [0x07, 0x03, 0x4e, 0x00]

        self.libmetawear.mbl_mw_ibeacon_set_major(self.board, 78);
        self.assertEqual(self.command, expected)

    def test_set_major_signal(self):
        expected= [
            [0x09, 0x02, 0x01, 0x01, 0xff, 0x00, 0x02, 0x13],
            [0x0a, 0x02, 0x09, 0x03, 0x00, 0x07, 0x03, 0x02, 0x09, 0x00],
            [0x0a, 0x03, 0x00, 0x00]
        ]

        switch= self.libmetawear.mbl_mw_switch_get_state_data_signal(self.board)

        self.libmetawear.mbl_mw_dataprocessor_counter_create_size(switch, 4, None, self.processor_handler)
        self.events["processor"].wait()

        self.libmetawear.mbl_mw_event_record_commands(self.processors[0])
        self.libmetawear.mbl_mw_ibeacon_set_major_signal(self.board, self.processors[0])
        self.libmetawear.mbl_mw_event_end_record(self.processors[0], None, self.commands_recorded_fn)
        self.events["event"].wait()

        self.assertEqual(self.command_history, expected)

    def test_set_minor(self):
        expected= [0x07, 0x04, 0x1d, 0x1d]

        self.libmetawear.mbl_mw_ibeacon_set_minor(self.board, 7453);
        self.assertEqual(self.command, expected)

    def test_set_period(self):
        expected= [0x07, 0x07, 0xb3, 0x3a]

        self.libmetawear.mbl_mw_ibeacon_set_period(self.board, 15027);
        self.assertEqual(self.command, expected)

    def test_set_rx_power(self):
        expected= [0x07, 0x05, 0xc9]

        self.libmetawear.mbl_mw_ibeacon_set_rx_power(self.board, -55);
        self.assertEqual(self.command, expected)

    def test_set_tx_power(self):
        expected= [0x07, 0x06, 0xf4]

        self.libmetawear.mbl_mw_ibeacon_set_tx_power(self.board, -12);
        self.assertEqual(self.command, expected)

    def test_set_uuid(self):
        expected= [0x07, 0x02, 0x5a, 0xe7, 0xba, 0xfb, 0x4c, 0x46, 0xdd, 0xd9, 0x95, 0x91, 0xcb, 0x85, 0x06, 0x90, 0x6a, 0x32]

        bytes = uuid.UUID('{326a9006-85cb-9195-d9dd-464cfbbae75a}').bytes[::-1]
        ad_uuid = cast(bytes, POINTER(c_ubyte * 16))
        self.libmetawear.mbl_mw_ibeacon_set_uuid(self.board, ad_uuid.contents)
        self.assertEqual(self.command, expected)

    def test_enable(self):
        expected= [0x07, 0x01, 0x01]

        self.libmetawear.mbl_mw_ibeacon_enable(self.board);
        self.assertEqual(self.command, expected)

    def test_ibeacon_disable(self):
        expected= [0x07, 0x01, 0x00]

        self.libmetawear.mbl_mw_ibeacon_disable(self.board);
        self.assertEqual(self.command, expected)



from common import TestMetaWearBase
from ctypes import create_string_buffer
from mbientlab.metawear.cbindings import *

class TestMultiChannelTemperatureMwr(TestMetaWearBase):
    def setUp(self):
        self.boardType= TestMetaWearBase.METAWEAR_R_BOARD

        super().setUp()

    def test_configure_ext_thermistor(self):
        expected= [0x04, 0x02, 0x01, 0x00, 0x01, 0x00]

        self.libmetawear.mbl_mw_multi_chnl_temp_configure_ext_thermistor(self.board, MetaWearRChannel.EXT_THERMISTOR, 0, 1, 0)
        self.assertListEqual(self.command, expected)

    def test_read_temperature(self):
        channels= [
            [[0x04, 0x81, 0x00], MetaWearRChannel.ON_DIE],
            [[0x04, 0x81, 0x01], MetaWearRChannel.EXT_THERMISTOR]
        ]

        for chnl in channels:
             with self.subTest(channel=chnl):
                temp_signal= self.libmetawear.mbl_mw_multi_chnl_temp_get_temperature_data_signal(self.board, chnl[1])
                self.libmetawear.mbl_mw_datasignal_subscribe(temp_signal, None, self.sensor_data_handler)
                self.libmetawear.mbl_mw_datasignal_read(temp_signal)
                self.assertListEqual(self.command, chnl[0])

    def test_read_temperature_silent(self):
        channels= [
            [[0x04, 0xc1, 0x00], MetaWearRChannel.ON_DIE],
            [[0x04, 0xc1, 0x01], MetaWearRChannel.EXT_THERMISTOR]
        ]

        for chnl in channels:
             with self.subTest(channel=chnl):
                temp_signal= self.libmetawear.mbl_mw_multi_chnl_temp_get_temperature_data_signal(self.board, chnl[1])
                self.libmetawear.mbl_mw_datasignal_read(temp_signal)
                self.assertListEqual(self.command, chnl[0])

    def test_get_temperature_data(self):
        responses= [
            [create_string_buffer(b'\x04\x81\x00\x00\x01', 5), 32.0, MetaWearRChannel.ON_DIE],
            [create_string_buffer(b'\x04\x81\x01\xac\x00', 5), 21.5, MetaWearRChannel.EXT_THERMISTOR]
        ]

        for resp in responses:
             with self.subTest(response=resp):
                temp_signal= self.libmetawear.mbl_mw_multi_chnl_temp_get_temperature_data_signal(self.board, resp[2])
                self.libmetawear.mbl_mw_datasignal_subscribe(temp_signal, None, self.sensor_data_handler)
                self.notify_mw_char(resp[0])
                self.assertAlmostEqual(self.data_float.value, resp[1])

    def test_get_num_channels(self):
        expected= 2
        result= self.libmetawear.mbl_mw_multi_chnl_temp_get_num_channels(self.board)

        self.assertEqual(result, expected)

    def test_get_source(self):
        channels= [TemperatureSource.NRF_DIE, TemperatureSource.EXT_THERM]

        for i in range(0, len(channels)):
            with self.subTest(channel=i):
                source= self.libmetawear.mbl_mw_multi_chnl_temp_get_source(self.board, i)
                self.assertEqual(source, channels[i])

    def test_get_invalid_source(self):
        channels= [-1, 2]

        for chnl in channels:
             with self.subTest(channel=chnl):
                self.assertEqual(TemperatureSource.INVALID, self.libmetawear.mbl_mw_multi_chnl_temp_get_source(self.board, chnl))

class TestMultiChannelTemperatureMwrPro(TestMetaWearBase):
    def setUp(self):
        self.boardType= TestMetaWearBase.METAWEAR_RPRO_BOARD
        super().setUp()

    def test_configure_ext_thermistor(self):
        expected= [0x04, 0x02, 0x02, 0x00, 0x01, 0x00]

        self.libmetawear.mbl_mw_multi_chnl_temp_configure_ext_thermistor(self.board, MetaWearRProChannel.EXT_THERMISTOR, 0, 1, 0)
        self.assertListEqual(self.command, expected)

    def test_read_temperature(self):
        channels= [
            [[0x04, 0x81, 0x00], MetaWearRProChannel.ON_DIE],
            [[0x04, 0x81, 0x01], MetaWearRProChannel.ON_BOARD_THERMISTOR],
            [[0x04, 0x81, 0x02], MetaWearRProChannel.EXT_THERMISTOR],
            [[0x04, 0x81, 0x03], MetaWearRProChannel.BMP280]
        ]

        for chnl in channels:
             with self.subTest(channel=chnl):
                temp_signal= self.libmetawear.mbl_mw_multi_chnl_temp_get_temperature_data_signal(self.board, chnl[1])
                self.libmetawear.mbl_mw_datasignal_subscribe(temp_signal, None, self.sensor_data_handler)
                self.libmetawear.mbl_mw_datasignal_read(temp_signal)
                self.assertListEqual(self.command, chnl[0])

    def test_read_temperature_silent(self):
        channels= [
            [[0x04, 0xc1, 0x00], MetaWearRProChannel.ON_DIE],
            [[0x04, 0xc1, 0x01], MetaWearRProChannel.ON_BOARD_THERMISTOR],
            [[0x04, 0xc1, 0x02], MetaWearRProChannel.EXT_THERMISTOR],
            [[0x04, 0xc1, 0x03], MetaWearRProChannel.BMP280]
        ]

        for chnl in channels:
             with self.subTest(channel=chnl):
                temp_signal= self.libmetawear.mbl_mw_multi_chnl_temp_get_temperature_data_signal(self.board, chnl[1])
                self.libmetawear.mbl_mw_datasignal_read(temp_signal)
                self.assertListEqual(self.command, chnl[0])

    def test_get_temperature_data(self):
        responses= [
            [create_string_buffer(b'\x04\x81\x00\xfe\x00', 5), 31.75, MetaWearRProChannel.ON_DIE],
            [create_string_buffer(b'\x04\x81\x01\xa8\x00', 5), 21.00, MetaWearRProChannel.ON_BOARD_THERMISTOR],
            [create_string_buffer(b'\x04\x81\x02\xac\xff', 5), -10.5, MetaWearRProChannel.EXT_THERMISTOR],
            [create_string_buffer(b'\x04\x81\x03\x00\x00', 5), 0, MetaWearRProChannel.BMP280]
        ]

        for resp in responses:
             with self.subTest(response=resp):
                temp_signal= self.libmetawear.mbl_mw_multi_chnl_temp_get_temperature_data_signal(self.board, resp[2])
                self.libmetawear.mbl_mw_datasignal_subscribe(temp_signal, None, self.sensor_data_handler)
                self.notify_mw_char(resp[0])
                self.assertAlmostEqual(self.data_float.value, resp[1])

    def test_get_num_channels(self):
        expected= 4
        result= self.libmetawear.mbl_mw_multi_chnl_temp_get_num_channels(self.board)

        self.assertEqual(result, expected)

    def test_get_source(self):
        channels= [TemperatureSource.NRF_DIE, TemperatureSource.PRESET_THERM, 
                TemperatureSource.EXT_THERM, TemperatureSource.BMP280]

        for i in range(0, len(channels)):
            with self.subTest(channel=i):
                source= self.libmetawear.mbl_mw_multi_chnl_temp_get_source(self.board, i)
                self.assertEqual(source, channels[i])

    def test_get_invalid_source(self):
        channels= [-1, 4]

        for chnl in channels:
             with self.subTest(channel=chnl):
                self.assertEqual(TemperatureSource.INVALID, self.libmetawear.mbl_mw_multi_chnl_temp_get_source(self.board, chnl))



from common import TestMetaWearBase
from ctypes import create_string_buffer
from mbientlab.metawear.cbindings import *

class BarometerBoschBase:
    class TestBarometerBoschConfig(TestMetaWearBase):
        def test_set_oversampling(self):
            tests= [
                {
                    'expected': [0x12, 0x03, 0x24, 0x00],
                    'oversampling': BaroBoschOversampling.ULTRA_LOW_POWER,
                    'oversampling_name': 'ultra low power'
                },
                {
                    'expected': [0x12, 0x03, 0x28, 0x00],
                    'oversampling': BaroBoschOversampling.LOW_POWER,
                    'oversampling_name': 'low power'
                },
                {
                    'expected': [0x12, 0x03, 0x2c, 0x00],
                    'oversampling': BaroBoschOversampling.STANDARD,
                    'oversampling_name': 'standard'
                },
                {
                    'expected': [0x12, 0x03, 0x30, 0x00],
                    'oversampling': BaroBoschOversampling.HIGH,
                    'oversampling_name': 'high'
                },
                {
                    'expected': [0x12, 0x03, 0x54, 0x00],
                    'oversampling': BaroBoschOversampling.ULTRA_HIGH,
                    'oversampling_name': 'ultra high'
                }
            ]

            for test in tests:
                with self.subTest(oversampling= test['oversampling_name']):
                    self.libmetawear.mbl_mw_baro_bosch_set_oversampling(self.board, test['oversampling'])
                    self.libmetawear.mbl_mw_baro_bosch_write_config(self.board)
                    self.assertEqual(self.command, test['expected'])

        def test_set_filter(self):
            tests= [
                {
                    'expected': [0x12, 0x03, 0x2c, 0x00],
                    'filter': BaroBoschIirFilter.OFF,
                    'iir_filter': 'off'
                },
                {
                    'expected': [0x12, 0x03, 0x2c, 0x04],
                    'filter': BaroBoschIirFilter.AVG_2,
                    'iir_filter': 'avg 2'
                },
                {
                    'expected': [0x12, 0x03, 0x2c, 0x08],
                    'filter': BaroBoschIirFilter.AVG_4,
                    'iir_filter': 'avg 4'
                },
                {
                    'expected': [0x12, 0x03, 0x2c, 0x0c],
                    'filter': BaroBoschIirFilter.AVG_8,
                    'iir_filter': 'avg 8'
                },
                {
                    'expected': [0x12, 0x03, 0x2c, 0x10],
                    'filter': BaroBoschIirFilter.AVG_16,
                    'iir_filter': 'avg 16'
                }
            ]

            for test in tests:
                with self.subTest(iir_filter= test['iir_filter']):
                    self.libmetawear.mbl_mw_baro_bosch_set_iir_filter(self.board, test['filter'])
                    self.libmetawear.mbl_mw_baro_bosch_write_config(self.board)
                    self.assertEqual(self.command, test['expected'])

        def test_start(self):
            expected= [0x12, 0x04, 0x01, 0x01]

            self.libmetawear.mbl_mw_baro_bosch_start(self.board);
            self.assertEqual(self.command, expected)

        def test_stop(self):
            expected= [0x12, 0x04, 0x00, 0x00]

            self.libmetawear.mbl_mw_baro_bosch_stop(self.board);
            self.assertEqual(self.command, expected)

    class TestBarometerBoschPressureData(TestMetaWearBase):
        def setUp(self):
            super().setUp()

            self.pa_data_signal= self.libmetawear.mbl_mw_baro_bosch_get_pressure_data_signal(self.board)

        def test_get_pressure_data(self):
            response= create_string_buffer(b'\x12\x01\xd3\x35\x8b\x01', 6)
            expected= 101173.828125

            self.libmetawear.mbl_mw_datasignal_subscribe(self.pa_data_signal, None, self.sensor_data_handler)
            self.notify_mw_char(response)
            self.assertAlmostEqual(self.data_float.value, expected)

        def test_pressure_subscribe(self):
            expected= [0x12, 0x1, 0x1]

            self.libmetawear.mbl_mw_datasignal_subscribe(self.pa_data_signal, None, self.sensor_data_handler)
            self.assertListEqual(self.command, expected)

        def test_pressure_unsubscribe(self):
            expected= [0x12, 0x1, 0x0]

            self.libmetawear.mbl_mw_datasignal_unsubscribe(self.pa_data_signal);
            self.assertListEqual(self.command, expected)

    class TestBarometerBoschAltitudeData(TestMetaWearBase):
        def setUp(self):
            super().setUp()

            self.m_data_signal= self.libmetawear.mbl_mw_baro_bosch_get_altitude_data_signal(self.board)

        def test_get_altitude_data(self):
            response= create_string_buffer(b'\x12\x02\x1e\x1f\xfe\xff', 6)
            expected= -480.8828125

            self.libmetawear.mbl_mw_datasignal_subscribe(self.m_data_signal, None, self.sensor_data_handler)
            self.notify_mw_char(response)
            self.assertAlmostEqual(self.data_float.value, expected)

        def test_altitude_subscribe(self):
            expected= [0x12, 0x2, 0x1]

            self.libmetawear.mbl_mw_datasignal_subscribe(self.m_data_signal, None, self.sensor_data_handler);
            self.assertListEqual(self.command, expected)

        def test_altitude_unsubscribe(self):
            expected= [0x12, 0x2, 0x0]

            self.libmetawear.mbl_mw_datasignal_unsubscribe(self.m_data_signal)
            self.assertListEqual(self.command, expected)



from common import TestMetaWearBase
from ctypes import byref, create_string_buffer
from mbientlab.metawear.cbindings import *

class TestGpioDigitalConfig(TestMetaWearBase):
    def test_set_digital_out(self):
        expected= [0x05, 0x01, 0x00]

        self.libmetawear.mbl_mw_gpio_set_digital_output(self.board, 0)
        self.assertEqual(self.command, expected)

    def test_clear_digital_out(self):
        expected= [0x05, 0x02, 0x01]

        self.libmetawear.mbl_mw_gpio_clear_digital_output(self.board, 1)
        self.assertEqual(self.command, expected)

    def test_set_pull_mode(self):
        tests= [
            {'expected': [0x05, 0x05, 0x02], 'mode': GpioPullMode.NONE, 'pin': 2},
            {'expected': [0x05, 0x04, 0x03], 'mode': GpioPullMode.DOWN, 'pin': 3},
            {'expected': [0x05, 0x03, 0x04], 'mode': GpioPullMode.UP, 'pin': 4}
        ]

        for test in tests:
            with self.subTest(pullmode=test['mode']):
                self.libmetawear.mbl_mw_gpio_set_pull_mode(self.board, test['pin'], test['mode'])
                self.assertEqual(self.command, test['expected'])

    def test_set_pin_change_type(self):
        tests= [
            {'expected': [0x05, 0x09, 0x05, 0x03], 'mode': GpioPinChangeType.ANY, 'pin': 5},
            {'expected': [0x05, 0x09, 0x06, 0x02], 'mode': GpioPinChangeType.FALLING, 'pin': 6},
            {'expected': [0x05, 0x09, 0x07, 0x01], 'mode': GpioPinChangeType.RISING, 'pin': 7}
        ]

        for test in tests:
            with self.subTest(changetype=test['mode']):
                self.libmetawear.mbl_mw_gpio_set_pin_change_type(self.board, test['pin'], test['mode'])
                self.assertEqual(self.command, test['expected'])

class TestGpioDigitalData(TestMetaWearBase):
    def test_pin_monitor(self):
        tests= [
            {'expected': 1, 'response': create_string_buffer(b'\x05\x0a\x00\x01', 4), 'change_type': "rising"},
            {'expected': 0, 'response': create_string_buffer(b'\x05\x0a\x00\x00', 4), 'change_type': "falling"}
        ]

        pin_monitor_signal= self.libmetawear.mbl_mw_gpio_get_pin_monitor_data_signal(self.board, 0);
        self.libmetawear.mbl_mw_datasignal_subscribe(pin_monitor_signal, None, self.sensor_data_handler)

        for test in tests:
            with self.subTest(changetype=test['change_type']):
                self.notify_mw_char(test['response'])
                self.assertEqual(self.data_uint32.value, test['expected'])

    # Combining subscribe and unsubscribe since unsubscribing from pin monitoring does not 
    # issue any metawear commands 
    def test_pin_monitor_subscribe(self):
        expected= [0x5, 0xa, 0x1]

        pin_monitor_signal= self.libmetawear.mbl_mw_gpio_get_pin_monitor_data_signal(self.board, 1);
        self.libmetawear.mbl_mw_datasignal_subscribe(pin_monitor_signal, None, self.sensor_data_handler)
        self.libmetawear.mbl_mw_datasignal_unsubscribe(pin_monitor_signal)
        self.assertEqual(self.command, expected)

    def test_pin_monitor_start(self):
        expected= [0x05, 0x0b, 0x05, 0x01]

        self.libmetawear.mbl_mw_gpio_start_pin_monitoring(self.board, 5)
        self.assertEqual(self.command, expected)

    def test_pin_monitor_stop(self):
        expected= [0x05, 0x0b, 0x06, 0x00]

        self.libmetawear.mbl_mw_gpio_stop_pin_monitoring(self.board, 6)
        self.assertEqual(self.command, expected)

    def test_read_digital_input(self):
        expected= [0x05, 0x88, 0x04]

        di_signal= self.libmetawear.mbl_mw_gpio_get_digital_input_data_signal(self.board, 4)
        self.libmetawear.mbl_mw_datasignal_subscribe(di_signal, None, self.sensor_data_handler)
        self.libmetawear.mbl_mw_datasignal_read(di_signal)
        self.assertEqual(self.command, expected)

    def test_read_digital_input_silent(self):
        expected= [0x05, 0xc8, 0x04]

        di_signal= self.libmetawear.mbl_mw_gpio_get_digital_input_data_signal(self.board, 4)
        self.libmetawear.mbl_mw_datasignal_read(di_signal)
        self.assertEqual(self.command, expected)


    def test_handle_digital_state(self):
        tests= [
            {'expected': 1, 'response': create_string_buffer(b'\x05\x88\x07\x01', 4), 'state': "high" },
            {'expected': 0, 'response': create_string_buffer(b'\x05\x88\x07\x00', 4), 'state': "low"}
        ]

        pin_monitor_signal= self.libmetawear.mbl_mw_gpio_get_digital_input_data_signal(self.board, 7);
        self.libmetawear.mbl_mw_datasignal_subscribe(pin_monitor_signal, None, self.sensor_data_handler)

        for test in tests:
            with self.subTest(changetype=test['state']):
                self.notify_mw_char(test['response'])
                self.assertEqual(self.data_uint32.value, test['expected'])

class TestGpioAnalogData(TestMetaWearBase):
    def test_read_analog_input(self):
        tests= [
            {'expected': [0x05, 0x86, 0x03], 'mode': GpioAnalogReadMode.ABS_REF, 'pin': 3},
            {'expected': [0x05, 0x87, 0x02], 'mode': GpioAnalogReadMode.ADC, 'pin': 2}
        ]

        for test in tests:
            with self.subTest(readmode=test['mode']):
                an_signal= self.libmetawear.mbl_mw_gpio_get_analog_input_data_signal(self.board, test['pin'], test['mode'])
                self.libmetawear.mbl_mw_datasignal_subscribe(an_signal, None, self.sensor_data_handler)
                self.libmetawear.mbl_mw_datasignal_read(an_signal)
                self.assertEqual(self.command, test['expected'])

    def test_read_analog_input_silent(self):
        tests= [
            {'expected': [0x05, 0xc6, 0x03], 'mode': GpioAnalogReadMode.ABS_REF, 'pin': 3},
            {'expected': [0x05, 0xc7, 0x02], 'mode': GpioAnalogReadMode.ADC, 'pin': 2}
        ]

        for test in tests:
            with self.subTest(readmode=test['mode']):
                an_signal= self.libmetawear.mbl_mw_gpio_get_analog_input_data_signal(self.board, test['pin'], test['mode'])
                self.libmetawear.mbl_mw_datasignal_read(an_signal)
                self.assertEqual(self.command, test['expected'])

    def test_read_analog_enhanced(self):
        parameters= GpioAnalogReadParameters()
        tests= [
            {'expected': [0x05, 0xc6, 0x03], 'mode': GpioAnalogReadMode.ABS_REF, 'pin': 3},
            {'expected': [0x05, 0xc7, 0x02], 'mode': GpioAnalogReadMode.ADC, 'pin': 2}
        ]

        for test in tests:
            with self.subTest(readmode=test['mode']):
                an_signal= self.libmetawear.mbl_mw_gpio_get_analog_input_data_signal(self.board, test['pin'], test['mode'])
                self.libmetawear.mbl_mw_datasignal_read_with_parameters(an_signal, byref(parameters))
                self.assertEqual(self.command, test['expected'])

    def test_handle_analog_data(self):
        tests= [
            {'expected': 882, 'response': create_string_buffer(b'\x05\x87\x01\x72\x03', 5), 'mode': GpioAnalogReadMode.ADC},
            {'expected': 2498, 'response': create_string_buffer(b'\x05\x86\x01\xc2\x09', 5), 'mode': GpioAnalogReadMode.ABS_REF}
        ]

        for test in tests:
            with self.subTest(readmode=test['mode']):
                pin_monitor_signal= self.libmetawear.mbl_mw_gpio_get_analog_input_data_signal(self.board, 1, test['mode']);
                self.libmetawear.mbl_mw_datasignal_subscribe(pin_monitor_signal, None, self.sensor_data_handler)

                self.notify_mw_char(test['response'])
                self.assertEqual(self.data_uint32.value, test['expected'])

class TestGpioEnhancedAnalogRead(TestMetaWearBase):
    def setUp(self):
        self.metawear_r_services[0x5]= create_string_buffer(b'\x05\x80\x00\x02', 4)
        super().setUp()

    def test_read_analog_no_parameters(self):
        tests= [
            {'expected': [0x05, 0xc6, 0x03, 0xff, 0xff, 0x00, 0xff], 'mode': GpioAnalogReadMode.ABS_REF, 'pin': 3},
            {'expected': [0x05, 0xc7, 0x02, 0xff, 0xff, 0x00, 0xff], 'mode': GpioAnalogReadMode.ADC, 'pin': 2}
        ]

        for test in tests:
            with self.subTest(readmode=test['mode']):
                an_signal= self.libmetawear.mbl_mw_gpio_get_analog_input_data_signal(self.board, test['pin'], test['mode'])
                self.libmetawear.mbl_mw_datasignal_read(an_signal)
                self.assertEqual(self.command, test['expected'])

    def test_read_analog_with_parameters(self):
        tests= [
            {'expected': [0x05, 0x86, 0x03, 0x01, 0x02, 0x02, 0x15], 'mode': GpioAnalogReadMode.ABS_REF, 'pin': 3},
            {'expected': [0x05, 0x87, 0x02, 0x01, 0x02, 0x02, 0x15], 'mode': GpioAnalogReadMode.ADC, 'pin': 2}
        ]

        parameters= GpioAnalogReadParameters(pullup_pin= 1, pulldown_pin= 2, virtual_pin= 0x15, delay_us= 10)

        for test in tests:
            with self.subTest(readmode=test['mode']):
                an_signal= self.libmetawear.mbl_mw_gpio_get_analog_input_data_signal(self.board, test['pin'], test['mode'])
                self.libmetawear.mbl_mw_datasignal_subscribe(an_signal, None, self.sensor_data_handler)
                self.libmetawear.mbl_mw_datasignal_read_with_parameters(an_signal, byref(parameters))
                self.assertEqual(self.command, test['expected'])

    def test_handle_analog_data(self):
        tests= [
            {'expected': 218, 'response': create_string_buffer(b'\x05\x87\x15\xda\x00', 5), 'mode': GpioAnalogReadMode.ADC},
            {'expected': 728, 'response': create_string_buffer(b'\x05\x86\x15\xd8\x02', 5), 'mode': GpioAnalogReadMode.ABS_REF}
        ]

        for test in tests:
            with self.subTest(readmode=test['mode']):
                virtual_signal= self.libmetawear.mbl_mw_gpio_get_analog_input_data_signal(self.board, 0x15, test['mode'])
                self.libmetawear.mbl_mw_datasignal_subscribe(virtual_signal, None, self.sensor_data_handler)

                self.notify_mw_char(test['response'])
                self.assertEqual(self.data_uint32.value, test['expected'])



# usage: python update_firmware.py [mac] [version](optional)
from mbientlab.metawear import MetaWear, libmetawear
from threading import Event

import sys

device = MetaWear(sys.argv[1])
device.connect()
print("Connected")

args = {
    'progress_handler': lambda p: print("upload: %d%%" % (p)),
}
if (len(sys.argv) >= 3):
    args['version'] = sys.argv[2]

e = Event()
result = []
def dfu_handler(err):
    result.append(err)
    e.set()

device.update_firmware_async(dfu_handler, **args)
e.wait()

if (result[0] != None):
    raise result[0]


from common import TestMetaWearBase, to_string_buffer
from ctypes import create_string_buffer
from mbientlab.metawear.cbindings import *
from threading import Event
import copy

class TestSensorFusion(TestMetaWearBase):
    def setUp(self):
        self.boardType = TestMetaWearBase.METAWEAR_MOTION_R_BOARD

        super().setUp()

    def test_received_data(self):
        tests = [
            {
                'expected' : CorrectedCartesianFloat(x = -3.3799, y = 15.9995, z = -15.9995, accuracy = Const.SENSOR_FUSION_CALIBRATION_ACCURACY_UNRELIABLE),
                'response' : create_string_buffer(b'\x19\x04\x20\x3e\x53\xc5\x0c\xfe\x79\x46\x0c\xfe\x79\xc6\x00', 15),
                'data' : SensorFusionData.CORRECTED_ACC,
                'name': 'corrected acc'
            },
            {
                'expected' : CorrectedCartesianFloat(x = 72.669, y = -56.096, z = 528.820, accuracy = Const.SENSOR_FUSION_CALIBRATION_ACCURACY_UNRELIABLE),
                'response' : create_string_buffer(b'\x19\x05\x7a\x56\x91\x42\xb4\x62\x60\xc2\x73\x34\x04\x44\x00', 15),
                'data' : SensorFusionData.CORRECTED_GYRO,
                'name': 'corrected gyro'
            },
            {
                'expected' : CorrectedCartesianFloat(x = 32.500, y = -14.800, z = 29.700, accuracy = Const.SENSOR_FUSION_CALIBRATION_ACCURACY_HIGH),
                'response' : create_string_buffer(b'\x19\x06\x00\x00\x02\x42\xcd\xcc\x6c\xc1\x9a\x99\xed\x41\x03', 15),
                'data' : SensorFusionData.CORRECTED_MAG,
                'name': 'corrected mag'
            },
            {
                'expected' : Quaternion(w = 0.940, x = -0.050, y = -0.154, z = -0.301),
                'response' : create_string_buffer(b'\x19\x07\x1b\x9b\x70\x3f\x8c\x5e\x4d\xbd\x07\x7f\x1d\xbe\x78\x02\x9a\xbe', 18),
                'data' : SensorFusionData.QUATERNION,
                'name': 'quaternion'
            },
            {
                'expected' : EulerAngles(heading = 24.747, pitch = -120.862, yaw = 24.747, roll = -33.046),
                'response' : create_string_buffer(b'\x19\x08\xb1\xf9\xc5\x41\x44\xb9\xf1\xc2\x1a\x2f\x04\xc2\xb1\xf9\xc5\x41', 18),
                'data' : SensorFusionData.EULER_ANGLE,
                'name': 'euler angle'
            },
            {
                'expected' : CartesianFloat(x = 0.042, y = 0.826, z = -0.562),
                'response' : create_string_buffer(b'\x19\x09\xee\x20\xd3\x3e\xb2\x93\x01\x41\x04\x59\xb0\xc0', 14),
                'data' : SensorFusionData.GRAVITY_VECTOR,
                'name': 'gravity'
            },
            {
                'expected' : CartesianFloat(x = 0.296, y = 1.439, z = -0.380),
                'response' : create_string_buffer(b'\x19\x0a\x2f\xca\x39\x40\x86\xd4\x61\x41\x80\x4c\x6e\xc0', 14),
                'data' : SensorFusionData.LINEAR_ACC,
                'name': 'linear acc'
            }
        ]

        for test in tests:
            with self.subTest(data = test['name']):
                signal = self.libmetawear.mbl_mw_sensor_fusion_get_data_signal(self.board, test['data'])
                self.libmetawear.mbl_mw_datasignal_subscribe(signal, None, self.sensor_data_handler)

                expected_value = test['expected']
                self.notify_mw_char(test['response'])

                self.assertEqual(self.data, expected_value)

    def test_sensor_control(self):
        data_sources = [
            SensorFusionData.CORRECTED_ACC, SensorFusionData.CORRECTED_GYRO, SensorFusionData.CORRECTED_MAG,
            SensorFusionData.QUATERNION, SensorFusionData.EULER_ANGLE,
            SensorFusionData.GRAVITY_VECTOR, SensorFusionData.LINEAR_ACC
        ]
        test_bases = [
            {
                'enable': 6,
                'mode': SensorFusionMode.NDOF,
                'name': 'ndof',
                'expected': [
                    [0x03, 0x02, 0x01, 0x00],
                    [0x13, 0x02, 0x01, 0x00],
                    [0x15, 0x02, 0x01, 0x00],
                    [0x03, 0x01, 0x01],
                    [0x13, 0x01, 0x01],
                    [0x15, 0x01, 0x01],
                    [0x19, 0x03, 0x00, 0x00],
                    [0x19, 0x01, 0x01],
                    [0x19, 0x01, 0x00],
                    [0x19, 0x03, 0x00, 0x7f],
                    [0x03, 0x01, 0x00],
                    [0x13, 0x01, 0x00],
                    [0x15, 0x01, 0x00],
                    [0x03, 0x02, 0x00, 0x01],
                    [0x13, 0x02, 0x00, 0x01],
                    [0x15, 0x02, 0x00, 0x01]
                ]
            },
            {
                'enable': 4,
                'mode': SensorFusionMode.IMU_PLUS,
                'name': 'imu_plus',
                'expected': [
                    [0x03, 0x02, 0x01, 0x00],
                    [0x13, 0x02, 0x01, 0x00],
                    [0x03, 0x01, 0x01],
                    [0x13, 0x01, 0x01],
                    [0x19, 0x03, 0x00, 0x00],
                    [0x19, 0x01, 0x01],
                    [0x19, 0x01, 0x00],
                    [0x19, 0x03, 0x00, 0x7f],
                    [0x03, 0x01, 0x00],
                    [0x13, 0x01, 0x00],
                    [0x03, 0x02, 0x00, 0x01],
                    [0x13, 0x02, 0x00, 0x01]
                ]
            },
            {
                'enable': 4,
                'mode': SensorFusionMode.COMPASS,
                'name': 'compass',
                'expected': [
                    [0x03, 0x02, 0x01, 0x00],
                    [0x15, 0x02, 0x01, 0x00],
                    [0x03, 0x01, 0x01],
                    [0x15, 0x01, 0x01],
                    [0x19, 0x03, 0x00, 0x00],
                    [0x19, 0x01, 0x01],
                    [0x19, 0x01, 0x00],
                    [0x19, 0x03, 0x00, 0x7f],
                    [0x03, 0x01, 0x00],
                    [0x15, 0x01, 0x00],
                    [0x03, 0x02, 0x00, 0x01],
                    [0x15, 0x02, 0x00, 0x01]
                ]
            },
            {
                'enable': 4,
                'mode': SensorFusionMode.M4G,
                'name': 'm4g',
                'expected': [
                    [0x03, 0x02, 0x01, 0x00],
                    [0x15, 0x02, 0x01, 0x00],
                    [0x03, 0x01, 0x01],
                    [0x15, 0x01, 0x01],
                    [0x19, 0x03, 0x00, 0x00],
                    [0x19, 0x01, 0x01],
                    [0x19, 0x01, 0x00],
                    [0x19, 0x03, 0x00, 0x7f],
                    [0x03, 0x01, 0x00],
                    [0x15, 0x01, 0x00],
                    [0x03, 0x02, 0x00, 0x01],
                    [0x15, 0x02, 0x00, 0x01]
                ]
            }
        ]

        tests= []
        for base in test_bases:
            for source in data_sources:
                test = copy.deepcopy(base)
                test['expected'][test['enable']][2] |= (0x1 << source)
                test['source'] = source
                tests.append(test)

        for test in tests:
            with self.subTest(mode = test['name']):
                self.command_history = []
                self.libmetawear.mbl_mw_sensor_fusion_clear_enabled_mask(self.board)

                self.libmetawear.mbl_mw_sensor_fusion_set_mode(self.board, test['mode'])
                self.libmetawear.mbl_mw_sensor_fusion_enable_data(self.board, test['source'])
                self.libmetawear.mbl_mw_sensor_fusion_start(self.board)
                self.libmetawear.mbl_mw_sensor_fusion_stop(self.board)

                self.assertEqual(test['expected'], self.command_history)

    def test_read_calibration(self):
        signal = self.libmetawear.mbl_mw_sensor_fusion_calibration_state_data_signal(self.board)

        self.assertIsNone(signal)

class TestSensorFusionRev1(TestMetaWearBase):
    def setUp(self):
        self.boardType = TestMetaWearBase.METAWEAR_MOTION_R_BOARD
        self.metawear_motion_r_services[0x19]= create_string_buffer(b'\x19\x80\x00\x01\x03\x00\x06\x00\x02\x00\x01\x00', 12)

        super().setUp()

    def test_read_calibration(self):
        signal = self.libmetawear.mbl_mw_sensor_fusion_calibration_state_data_signal(self.board)

        self.libmetawear.mbl_mw_datasignal_subscribe(signal, None, self.sensor_data_handler)
        self.libmetawear.mbl_mw_datasignal_read(signal)
        self.assertEqual([0x19, 0x8b], self.command)


        expected_state = CalibrationState(
                accelerometer = Const.SENSOR_FUSION_CALIBRATION_ACCURACY_UNRELIABLE, 
                gyroscope = Const.SENSOR_FUSION_CALIBRATION_ACCURACY_LOW, 
                magnetometer = Const.SENSOR_FUSION_CALIBRATION_ACCURACY_MEDIUM
        )
        self.notify_mw_char(to_string_buffer([0x19, 0x8b, 0x00, 0x01, 0x02]))
        self.assertEqual(expected_state, self.data)

    def test_read_calibration_data(self):
        e = Event()
        actual = []
        def received(board, ctx, ptr):
            actual.append(ptr)
            e.set()

        fn_wrapper = FnVoid_VoidP_VoidP_CalibrationDataP(received)
        self.libmetawear.mbl_mw_sensor_fusion_read_calibration_data(self.board, None, fn_wrapper)
        e.wait()

        # unsupported for rev1
        self.assertFalse(bool(actual[0]))

    def test_write_calibration_data(self):
        data = CalibrationData(
            acc = (c_ubyte * 10)(),
            gyro = (c_ubyte * 10)(),
            mag = (c_ubyte * 10)()
        )

        self.libmetawear.mbl_mw_sensor_fusion_write_calibration_data(self.board, byref(data))

        # unsupported in rev 1, no commands
        self.assertEqual([], self.command_history)

class TestSensorFusionRev2(TestMetaWearBase):
    def setUp(self):
        self.boardType = TestMetaWearBase.METAWEAR_MOTION_R_BOARD
        self.metawear_motion_r_services[0x19]= create_string_buffer(b'\x19\x80\x00\x02\x03\x00\x06\x00\x02\x00\x01\x00', 12)

        super().setUp()

    def commandLogger(self, context, board, writeType, characteristic, command, length):
        prev = len(self.full_history)
        TestMetaWearBase.commandLogger(self, context, board, writeType, characteristic, command, length)
        curr = len(self.full_history)

        if (prev != curr):
            response = None
            if (command[0] == 0x19 and command[1] == 0x8c):
                response = to_string_buffer([0x19, 0x8c, 0xf6, 0xff, 0x00, 0x00, 0x0a, 0x00, 0xe8, 0x03, 0x03, 0x00])
            elif (command[0] == 0x19 and command[1] == 0x8d):
                response = to_string_buffer([0x19, 0x8d, 0x04, 0x00, 0x08, 0x00, 0x01, 0x00, 0x00, 0x00, 0x03, 0x00])
            elif(command[0] == 0x19 and command[1] == 0x8e):
                response = to_string_buffer([0x19, 0x8e, 0x66, 0x00, 0x17, 0xfd, 0x8a, 0xfc, 0x7f, 0x03, 0x01, 0x00])

            if (response != None):
                self.schedule_response(response)

    def test_read_calibration_data(self):
        expected = [
            [0xf6, 0xff, 0x00, 0x00, 0x0a, 0x00, 0xe8, 0x03, 0x03, 0x00],
            [0x04, 0x00, 0x08, 0x00, 0x01, 0x00, 0x00, 0x00, 0x03, 0x00],
            [0x66, 0x00, 0x17, 0xfd, 0x8a, 0xfc, 0x7f, 0x03, 0x01, 0x00]
        ]

        e = Event()
        actual = []
        def received(board, ctx, ptr):
            value = cast(ptr, POINTER(CalibrationData)).contents
            actual.append([value.acc[i] for i in range(0, 10)])
            actual.append([value.gyro[i] for i in range(0, 10)])
            actual.append([value.mag[i] for i in range(0, 10)])
            
            self.libmetawear.mbl_mw_memory_free(ptr)
            e.set()

        fn_wrapper = FnVoid_VoidP_VoidP_CalibrationDataP(received)
        self.libmetawear.mbl_mw_sensor_fusion_read_calibration_data(self.board, None, fn_wrapper)
        e.wait()

        self.assertEqual(expected, actual)

    def test_write_calibration_data(self):
        data = CalibrationData(
            acc = (c_ubyte * 10)(0xf6, 0xff, 0x00, 0x00, 0x0a, 0x00, 0xe8, 0x03, 0x03, 0x00),
            gyro = (c_ubyte * 10)(0x04, 0x00, 0x08, 0x00, 0x01, 0x00, 0x00, 0x00, 0x03, 0x00),
            mag = (c_ubyte * 10)(0x66, 0x00, 0x17, 0xfd, 0x8a, 0xfc, 0x7f, 0x03, 0x01, 0x00)
        )
        
        acc_cmd = [0x19, 0x0c, 0xf6, 0xff, 0x00, 0x00, 0x0a, 0x00, 0xe8, 0x03, 0x03, 0x00]
        gyro_cmd = [0x19, 0x0d, 0x04, 0x00, 0x08, 0x00, 0x01, 0x00, 0x00, 0x00, 0x03, 0x00]
        mag_cmd = [0x19, 0x0e, 0x66, 0x00, 0x17, 0xfd, 0x8a, 0xfc, 0x7f, 0x03, 0x01, 0x00]
        

        tests = [
            {
                'mode': SensorFusionMode.NDOF,
                'name': 'ndof',
                'expected': [
                    acc_cmd,
                    gyro_cmd,
                    mag_cmd
                ]
            },
            {
                'mode': SensorFusionMode.IMU_PLUS,
                'name': 'imu_plus',
                'expected': [
                    acc_cmd,
                    gyro_cmd
                ]
            },
            {
                'mode': SensorFusionMode.COMPASS,
                'name': 'compass',
                'expected': [
                    acc_cmd,
                    mag_cmd
                ]
            },
            {
                'mode': SensorFusionMode.M4G,
                'name': 'm4g',
                'expected': [
                    acc_cmd,
                    mag_cmd,
                ]
            }
        ]
        for test in tests:
            with self.subTest(mode = test['name']):
                self.command_history = []

                self.libmetawear.mbl_mw_sensor_fusion_set_mode(self.board, test['mode'])
                self.libmetawear.mbl_mw_sensor_fusion_write_calibration_data(self.board, byref(data))

                self.assertEqual(test['expected'], self.command_history)


from common import TestMetaWearBase

class TestHaptic(TestMetaWearBase):
    def test_start_motor(self):
        expected= [0x08, 0x01, 0xf8, 0x88, 0x13, 0x00]

        self.libmetawear.mbl_mw_haptic_start_motor(self.board, 100.0, 5000)
        self.assertListEqual(self.command, expected)

    def test_start_buzzer(self):
        expected= [0x08, 0x01, 0x7f, 0x4c, 0x1d, 0x01]

        self.libmetawear.mbl_mw_haptic_start_buzzer(self.board, 7500)
        self.assertListEqual(self.command, expected)



from common import TestMetaWearBase
from ctypes import create_string_buffer

class TestTimer(TestMetaWearBase):
    def test_create_timer(self):
        expected= [0x0c, 0x02, 0x45, 0x0c, 0x00, 0x00, 0x3B, 0x0, 0x0]

        self.libmetawear.mbl_mw_timer_create(self.board, 3141, 59, 1, None, self.timer_signal_ready)
        self.events["timer"].wait()

        self.assertEqual(self.command, expected)

    def test_create_timer_indefinite(self):
        expected= [0x0c, 0x02, 0xe8, 0x03, 0x00, 0x00, 0xff, 0xff, 0x01]

        self.libmetawear.mbl_mw_timer_create_indefinite(self.board, 1000, 0, None, self.timer_signal_ready)
        self.events["timer"].wait()

        self.assertEqual(self.command, expected)

    def test_start(self):
        expected= [0x0c, 0x03, 0x0]

        self.libmetawear.mbl_mw_timer_create(self.board, 1000, 10, 0, None, self.timer_signal_ready)
        self.events["timer"].wait()
        self.libmetawear.mbl_mw_timer_start(self.timerSignals[0])

        self.assertEqual(self.command, expected)

    def test_stop(self):
        expected= [0x0c, 0x04, 0x5]

        self.timerId= 5
        self.libmetawear.mbl_mw_timer_create(self.board, 1000, -1, 0, None, self.timer_signal_ready)
        self.events["timer"].wait()

        self.libmetawear.mbl_mw_timer_stop(self.timerSignals[0])

        self.assertEqual(self.command, expected)

    def test_remove(self):
        expected_cmds= [
            [0x0c, 0x05, 0x7],
            [0x0a, 0x04, 0x0],
            [0x0a, 0x04, 0x1]
        ]

        di_signal= self.libmetawear.mbl_mw_gpio_get_digital_input_data_signal(self.board, 3)
        temp_signal= self.libmetawear.mbl_mw_multi_chnl_temp_get_temperature_data_signal(self.board, 1)

        self.timerId= 7
        self.libmetawear.mbl_mw_timer_create(self.board, 667408, -1, 0, None, self.timer_signal_ready)
        self.events["timer"].wait()

        self.libmetawear.mbl_mw_event_record_commands(self.timerSignals[0])
        self.libmetawear.mbl_mw_datasignal_read(di_signal)
        self.libmetawear.mbl_mw_datasignal_read(temp_signal)
        self.libmetawear.mbl_mw_event_end_record(self.timerSignals[0], None, self.commands_recorded_fn)
        self.events["event"].wait()

        self.libmetawear.mbl_mw_timer_remove(self.timerSignals[0])

        # Ignore the add timer and events commands
        del self.command_history[0:5]
        self.assertEqual(self.command_history, expected_cmds)

class TestTimerTimeout(TestMetaWearBase):
    def commandLogger(self, context, board, writeType, characteristic, command, length):
        if (command[0] == 0xc and command[1] == 0x2):
            self.notify_mw_char(create_string_buffer(b'\x0c\x00', 2))
        else:
            super().commandLogger(context, board, writeType, characteristic, command, length)

    def test_timeout(self):
        self.libmetawear.mbl_mw_timer_create(self.board, 667408, -1, 0, None, self.timer_signal_ready)
        self.events["timer"].wait()

        self.assertIsNone(self.timerSignals[0])



from distutils.command.clean import clean
from multiprocessing import cpu_count
from shutil import copy2, move
from subprocess import call, STDOUT
from setuptools import setup
from setuptools.command.build_py import build_py

import os
import platform
import sys

machine = os.environ['MACHINE'] if 'MACHINE' in os.environ else ("arm" if "arm" in platform.machine() else ("x64" if sys.maxsize > 2**32 else "x86"))
root = os.path.dirname(os.path.abspath(__file__))
dest = os.path.join("mbientlab", "metawear")

class MetaWearClean(clean):
    def run(self):
        bindings = os.path.join(dest, "cbindings.py")
        if os.path.isfile(bindings):
            os.remove(bindings)

        if (platform.system() == 'Windows'):
            dll = os.path.join(dest, "MetaWear.Win32.dll")
            if os.path.isfile(dll):
                os.remove(dll)
        elif (platform.system() == 'Linux'):
            for f in os.listdir(dest):
                if (f.startswith("libmetawear.so")):
                    os.remove(os.path.join(dest, f))

class MetaWearBuild(build_py):
    @staticmethod
    def _move(src, dest, basename):
        for f in os.listdir(src):
            if (f.startswith(basename)):
                move(os.path.join(src, f), dest)

    def run(self):        
        cpp_sdk = os.path.join(root, 'MetaWear-SDK-Cpp')
        system = platform.system()
        dist_dir = os.path.join(cpp_sdk, 'dist', 'release', 'lib', "Win32" if machine == "x86" and system == "Windows" else machine)

        if os.path.exists(os.path.join(root, '.git')):
            status = call(["git", "submodule", "update", "--init"], cwd=root, stderr=STDOUT)
            if (status != 0):
                raise RuntimeError("Could not init git submodule")

        if (system == 'Windows'):
            if (call(["MSBuild.exe", "MetaWear.Win32.vcxproj", "/p:Platform=%s" % machine, "/p:Configuration=Release"], cwd=cpp_sdk, stderr=STDOUT) != 0):
                raise RuntimeError("Failed to compile MetaWear.dll")

            move(os.path.join(dist_dir, "MetaWear.Win32.dll"), dest)
        elif (system == 'Linux'):
            status = call(["make", "-C", "MetaWear-SDK-Cpp", "OPT_FLAGS=-Wno-strict-aliasing", "-j%d" % (cpu_count())], cwd=root, stderr=STDOUT)
            if (status != 0):
                raise RuntimeError("Failed to compile C++ SDK")

            MetaWearBuild._move(dist_dir, dest, 'libmetawear.so')
        else:
            raise RuntimeError("MetaWear Python SDK not supported for '%s'" % platform.system())

        copy2(os.path.join(cpp_sdk, 'bindings', 'python', 'mbientlab', 'metawear', 'cbindings.py'), dest)
        build_py.run(self)

so_pkg_data = ['libmetawear.so'] if platform.system() == 'Linux' else ['MetaWear.Win32.dll']
setup(
    name='metawear',
    packages=['mbientlab', 'mbientlab.metawear'],
    version='0.6.1',
    description='Python bindings for the MetaWear C++ SDK by MbientLab',
    long_description=open(os.path.join(os.path.dirname(__file__), "README.rst")).read(),
    package_data={'mbientlab.metawear': so_pkg_data},
    include_package_data=True,
    url='https://github.com/mbientlab/MetaWear-SDK-Python',
    author='MbientLab',
    author_email="hello@mbientlab.com",
    install_requires=[
        'warble >= 1.1, < 2.0',
        'requests'
    ],
    cmdclass={
        'build_py': MetaWearBuild,
        'clean': MetaWearClean
    },
    keywords = ['sensors', 'mbientlab', 'metawear', 'bluetooth le', 'native'],
    python_requires='>=2.7',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Operating System :: POSIX :: Linux',
        'Operating System :: Microsoft :: Windows :: Windows 10',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
    ]
)



from common import TestMetaWearBase
from ctypes import create_string_buffer
from mbientlab.metawear.cbindings import *
import threading

class TestEvent(TestMetaWearBase):
    def test_schedule_read_temp(self):
        expected_cmds= [
            [0x0c, 0x02, 0x64, 0xb0, 0x05, 0xa2, 0x50, 0xb3, 0x1],
            [0x0a, 0x02, 0x0c, 0x06, 0x00, 0x04, 0xc1, 0x01],
            [0x0a, 0x03, 0x01]
        ]

        temp_signal= self.libmetawear.mbl_mw_multi_chnl_temp_get_temperature_data_signal(self.board, 1)
        self.libmetawear.mbl_mw_timer_create(self.board, 2718281828, 45904, 0, None, self.timer_signal_ready)
        self.events["timer"].wait()

        self.libmetawear.mbl_mw_event_record_commands(self.timerSignals[0])
        self.libmetawear.mbl_mw_datasignal_read(temp_signal)
        self.libmetawear.mbl_mw_event_end_record(self.timerSignals[0], None, self.commands_recorded_fn)
        self.events["event"].wait()

        self.assertEqual(self.command_history, expected_cmds)

    def test_schedule_read_gpio_adc(self):
        expected_cmds= [
            [0x0c, 0x02, 0x10, 0x2f, 0x0a, 0x00, 0xff, 0xff, 0x0],
            [0x0a, 0x02, 0x0c, 0x06, 0x01, 0x05, 0xc7, 0x01],
            [0x0a, 0x03, 0x02]
        ]
        adc_signal= self.libmetawear.mbl_mw_gpio_get_analog_input_data_signal(self.board, 2, GpioAnalogReadMode.ADC)

        self.timerId= 1
        self.libmetawear.mbl_mw_timer_create_indefinite(self.board, 667408, 1, None, self.timer_signal_ready)
        self.events["timer"].wait()

        self.libmetawear.mbl_mw_event_record_commands(self.timerSignals[0])
        self.libmetawear.mbl_mw_datasignal_read(adc_signal)
        self.libmetawear.mbl_mw_event_end_record(self.timerSignals[0], None, self.commands_recorded_fn)
        self.events["event"].wait()

        self.assertEqual(self.command_history, expected_cmds)

    def test_schedule_read_temp_gpio(self):
        expected_cmds= [
            [0x0c, 0x02, 0x96, 0x22, 0x0, 0x0, 0x5e, 0x49, 0x0],
            [0x0c, 0x02, 0xda, 0x79, 0x7d, 0x07, 0xff, 0xff, 0x1],
            [0x0a, 0x02, 0x0c, 0x06, 0x0, 0x05, 0xc6, 0x01],
            [0x0a, 0x03, 0x03],
            [0x0a, 0x02, 0x0c, 0x06, 0x01, 0x04, 0xc1, 0x01],
            [0x0a, 0x03, 0x0]
        ]

        self.libmetawear.mbl_mw_timer_create(self.board, 8854, 18782, 1, None, self.timer_signal_ready)
        self.events["timer"].wait()

        self.events["timer"].clear()
        self.libmetawear.mbl_mw_timer_create_indefinite(self.board, 125663706, 0, None, self.timer_signal_ready)
        self.events["timer"].wait()

        abs_ref_signal= self.libmetawear.mbl_mw_gpio_get_analog_input_data_signal(self.board, 3, GpioAnalogReadMode.ABS_REF)
        self.libmetawear.mbl_mw_event_record_commands(self.timerSignals[0])
        self.libmetawear.mbl_mw_datasignal_read(abs_ref_signal)
        self.libmetawear.mbl_mw_event_end_record(self.timerSignals[0], None, self.commands_recorded_fn)
        self.events["event"].wait()

        self.events["event"].clear()
        temp_signal= self.libmetawear.mbl_mw_multi_chnl_temp_get_temperature_data_signal(self.board, 0)
        self.libmetawear.mbl_mw_event_record_commands(self.timerSignals[1])
        self.libmetawear.mbl_mw_datasignal_read(temp_signal)
        self.libmetawear.mbl_mw_event_end_record(self.timerSignals[1], None, self.commands_recorded_fn)
        self.events["event"].wait()

        self.assertEqual(self.command_history, expected_cmds)

class TestEventTimeout(TestMetaWearBase):
    def commandLogger(self, context, board, writeType, characteristic, command, length):
        if (command[0] == 0xa and command[1] == 0x3):
            self.notify_mw_char(create_string_buffer(b'\x0a\x00', 2))
        else:
            super().commandLogger(context, board, writeType, characteristic, command, length)

    def test_timeout(self):
        self.e= threading.Event()

        signal= self.libmetawear.mbl_mw_switch_get_state_data_signal(self.board)

        abs_ref_signal= self.libmetawear.mbl_mw_gpio_get_analog_input_data_signal(self.board, 3, GpioAnalogReadMode.ABS_REF)
        self.libmetawear.mbl_mw_event_record_commands(signal)
        self.libmetawear.mbl_mw_datasignal_read(abs_ref_signal)
        self.libmetawear.mbl_mw_event_end_record(signal, None, self.commands_recorded_fn)
        self.events["event"].wait()

        self.assertEqual(self.event_status[0], Const.STATUS_ERROR_TIMEOUT)
